/*
 * This file is part of Alida, a Java library for 
 * Advanced Library for Integrated Development of Data Analysis Applications.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on Alida, visit
 *
 *    http://www.informatik.uni-halle.de/alida/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5959 $
 * $Date: 2012-09-12 16:21:51 +0200 (Mi, 12. Sep 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.Alida.gui;

import java.awt.event.ActionEvent;

import javax.swing.JOptionPane;



import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerSwing.ProviderInteractionLevel;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDProcessingDAGException;
import de.unihalle.informatik.Alida.operator.*;

/**
 * Threaded class for executing an operator.
 * 
 * @author Stefan Posch
 * @author Birgit Moeller
 */

public class ALDExecuteOperator implements Runnable {

	/**
	 * Private flag for debug output.
	 */
	@SuppressWarnings("unused")
	private boolean debug = false;

	/**
	 * Operator to run.
	 */
	private ALDOperator op = null;
	
	/**
	 * Flag to indicate if result window should show-up.
	 */
	private boolean showResultsAfterTermination = false;

	/**
	 * Associated configuration/control frame.
	 */
	private ALDOperatorControlFrame controlFrame;

	// ...
	private ALDExecuteOperatorResultFrame resultFrame;

	/**
	 * Interaction level the providers used in result visualization should obey.
	 */
	private ProviderInteractionLevel pLevel = 
			ProviderInteractionLevel.ALL_ALLOWED;
	
	// ...
	public ALDExecuteOperatorResultFrame getResultFrame() {
		return this.resultFrame;
	}

	/**
	 * Default constructor.
	 */
	public ALDExecuteOperator() {
		// nothing to do here
	}

	/**
	 * Initialization function.
	 * @param o															Operator to be executed.
	 * @param cFrame 												Corresponding control frame.
	 * @param _showResultsAfterTermination	If true, results pop-up immediately.
	 * @param _pLevel 	Interaction level for providers.
	 */
	public void init(ALDOperator o, ALDOperatorControlFrame cFrame,
				boolean _showResultsAfterTermination,
				ProviderInteractionLevel _pLevel) {
		this.op = o;
		this.controlFrame = cFrame;
		this.showResultsAfterTermination = _showResultsAfterTermination;
		this.pLevel = _pLevel;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {

		try {
			this.op.runOp(false);
			// after operator is finished, show result frame
			this.resultFrame = new ALDExecuteOperatorResultFrame(this.op,
																							this.controlFrame, this.pLevel);
			if (this.showResultsAfterTermination)
				this.resultFrame.setVisible(true);
			// notify controller
			this.controlFrame.actionPerformed(new ActionEvent(this, 1,
					"threadstop"));
		} catch (ALDOperatorException ex) {
			if (ex.getMessage().contains("argument validation failed")) {
				JOptionPane.showMessageDialog(null,
						"Operator validation failed,\n check your parameters!",
						"Error!", JOptionPane.ERROR_MESSAGE);
				this.controlFrame.actionPerformed(new ActionEvent(this, 1,
						"threadstop"));
			} else {
				ex.printStackTrace();
				// notify controller
				this.controlFrame.actionPerformed(new ActionEvent(this,1,"threadstop"));
			}
		} catch (ALDProcessingDAGException ex) {
			System.err.println(ex.getMessage());
			ex.printStackTrace();
		}
	}
}
