package de.unihalle.informatik.Alida.gui.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Locale;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.SwingConstants;

import org.w3c.dom.Document;

import com.mxgraph.io.mxCodec;
import com.mxgraph.io.mxCodecRegistry;
import com.mxgraph.io.mxObjectCodec;
import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGeometry;
import com.mxgraph.model.mxICell;
import com.mxgraph.swing.mxGraphComponent;
import com.mxgraph.swing.handler.mxConnectionHandler;
import com.mxgraph.util.mxEvent;
import com.mxgraph.util.mxEventObject;
import com.mxgraph.util.mxEventSource.mxIEventListener;
import com.mxgraph.util.mxPoint;
import com.mxgraph.util.mxRectangle;
import com.mxgraph.util.mxUtils;
//import com.mxgraph.util.mxXmlUtils;
import com.mxgraph.view.mxGraph;
import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.alias.CannotResolveClassException;
import com.thoughtworks.xstream.io.xml.DomDriver;



import de.unihalle.informatik.Alida.annotations.ALDAOperator;
import de.unihalle.informatik.Alida.annotations.ALDAOperator.Level;
import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.Alida.annotations.Parameter.Direction;
import de.unihalle.informatik.Alida.dataio.ALDDataIOManagerSwing.ProviderInteractionLevel;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDProcessingDAGException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException.OperatorExceptionType;
import de.unihalle.informatik.Alida.gui.ALDChooseOpNameFrame;
import de.unihalle.informatik.Alida.gui.ALDOperatorChooserTree;
import de.unihalle.informatik.Alida.gui.ALDOperatorChooserTreeNode;
import de.unihalle.informatik.Alida.gui.ALDOperatorControlFrame;
import de.unihalle.informatik.Alida.helpers.ALDClassInfo;
import de.unihalle.informatik.Alida.operator.ALDOpParameterDescriptor;
import de.unihalle.informatik.Alida.operator.ALDOperator;
import de.unihalle.informatik.Alida.operator.ALDParameterDescriptor;

/**
 * Main Window for graphical programming.
 * 
 * @author Steven Kirchner
 * 
 */


@ALDAOperator(genericExecutionMode = ALDAOperator.ExecutionMode.NONE, 
	level = Level.APPLICATION)
public class ALDEditorFrame extends ALDOperator implements ActionListener {
	
	/** Extension added to filename which holds the external XML representation
	 * of the work flow to construct the filename of the external XML representation
	 * of the operator configurations
	 */
	public static String opsXMLFileExtension = "ops.xml";

	/**
	 * Main window for interactions.
	 */
	protected JFrame mainWindow;

	/**
	 * Builds the operator tree.
	 */
	protected ALDOperatorChooserTree opTree;

	/**
	 * Scroll pane displaying the operator tree.
	 */
	protected JScrollPane opTreePane;

	/**
	 * Graph where the nodes represents operators.
	 */
	private ALDWorkflowGraph opGraph;

	/**
	 * Component which holds the graph.
	 */
	protected mxGraphComponent graphComponent;
//	protected myGraphComponent graphComponent;

	// ...
	private HashMap<String, mxCell> nodeMap = new HashMap<String, mxCell>();

	protected HashMap<mxCell, ALDOperatorDescriptor> opDescrMap = new HashMap<mxCell, ALDOperatorDescriptor>();

	// TODO
	protected String tempOpNamePath = null;
	protected String tempOpName = null;
	private boolean runOnlyNode = false;

	// TODO Birgit - Comments
	protected JDialog loadMessageFrame;
	protected JLabel messageString;
	protected JButton okButton;
	
	protected LinkedList<mxCell> nodesToBeExecuted = new LinkedList<mxCell>(); 
  protected boolean workFlowLoaded = false;
	
	/**
	 * Default constructor
	 * 
	 * @throws ALDOperatorException
	 */
	public ALDEditorFrame() throws ALDOperatorException {
		// global language settings
		Locale.setDefault(Locale.ENGLISH);
		JComponent.setDefaultLocale(Locale.ENGLISH);
	}

	@Override
	protected void operate() throws ALDOperatorException,
			ALDProcessingDAGException {

		// set up main window
		mainWindow = new JFrame("ALDEditorFrame: NewGraph");
		mainWindow.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		mainWindow.setSize(500, 500);

		// build operator tree
		Collection<String> standardOps = 
				this.configureCollectionStandardOps();
		Collection<String> applicationOps = 
				this.configureCollectionApplicationOps();
		this.opTree = new ALDOperatorChooserTree(standardOps, applicationOps);

		// add the listener for mouse actions appearing in the tree
		this.opTree.addMouseListener(new TreeMouseAdapter(this));
		this.opTreePane = new JScrollPane(this.opTree);
		this.opTreePane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		
		// add tree to the window
		mainWindow.add(BorderLayout.WEST, this.opTreePane);

		// initialize the graph for the operators
		opGraph = new ALDWorkflowGraph() {
			@Override
      public String getToolTipForCell(Object cell) {
				mxCell port = (mxCell) cell;
				if (port.isVertex() && port.isConnectable()) {
					ALDOperatorDescriptor opDescr = opDescrMap.get(nodeMap
							.get(port.getParent().getValue().toString()));
					ALDParameterDescriptor parDescr = opDescr
							.getParameterDescriptor(((ALDPort) port.getValue())
									.getParName());
					String toolTip = "<html>" + "Name: " + parDescr.getLabel()
							+ "<br>" + "Class: "
							+ parDescr.getMyclass().getSimpleName() + "<br>"
							+ parDescr.getExplanation() + "</html>";
					return toolTip;
				}

				return null;
			}

			@Override
      public boolean isCellFoldable(Object cell, boolean collapse) {
				return false;
			}
		};
		EdgeConnectListener elist = new EdgeConnectListener();
		opGraph.setMinimumGraphSize(new mxRectangle(0, 0, 500, 500));
		opGraph.setAllowDanglingEdges(false);
		opGraph.setCellsEditable(false);
		opGraph.addListener(mxEvent.CONNECT_CELL, elist);

		// add graph to the component and finally to the panel
		graphComponent = new mxGraphComponent(opGraph);
//		graphComponent = new myGraphComponent(opGraph);
//		graphComponent.getGraphHandler().setMoveEnabled(true);
		graphComponent.getGraphControl().addMouseListener(
				new GraphMouseAdapter(this));
//		graphComponent.getConnectionHandler().addListener(mxEvent.ADD,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.ADD_CELLS,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.ADD_OVERLAY,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.BEGIN_UPDATE,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.CELLS_ADDED,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.CELLS_FOLDED,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.CELLS_MOVED,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.CELLS_ORDERED,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.DONE,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.DOWN,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.END_UPDATE,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.CELL_CONNECTED,elist);
		graphComponent.getConnectionHandler().addListener(mxEvent.CONNECT,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.CHANGE,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.FLIP_EDGE,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.MOVE_CELLS,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.TOGGLE_CELLS,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.NOTIFY,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.FIRED,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.LABEL_CHANGED,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.LAYOUT_CELLS,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.PAINT,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.REPAINT,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.SPLIT_EDGE,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.START_EDITING,elist);
//		graphComponent.getConnectionHandler().addListener(mxEvent.STOP,elist);
		graphComponent.getConnectionHandler().addListener(mxEvent.CONNECT_CELL,elist);

//		HIER!!!
//		graphComponent.getConnectionHandler().mouseReleased(arg0)
		
		graphComponent.setFocusable(true);
		graphComponent.setToolTips(true);

		mainWindow.getContentPane().add(graphComponent);

		// add a nice menu bar
		JMenuBar mainWindowMenu = new JMenuBar();

		JMenu fileM = new JMenu("File");
		JMenuItem newItem = new JMenuItem("New");
		newItem.setActionCommand("new");
		newItem.addActionListener(this);
		JMenuItem saveItem = new JMenuItem("Save");
		saveItem.setActionCommand("save");
		saveItem.addActionListener(this);
		JMenuItem loadItem = new JMenuItem("Load");
		loadItem.setActionCommand("load");
		loadItem.addActionListener(this);
		JMenuItem quitItem = new JMenuItem("Quit");
		quitItem.setActionCommand("quit");
		quitItem.addActionListener(this);
		fileM.add(newItem);
		fileM.add(saveItem);
		fileM.add(loadItem);
		fileM.add(quitItem);

		JMenu opSelectM = new JMenu("Operator Level");
		ButtonGroup operatorLevelGroup = new ButtonGroup();
		JRadioButtonMenuItem radioItemApplication = new JRadioButtonMenuItem(
				"Application");
		radioItemApplication.setActionCommand("viewApps");
		radioItemApplication.addActionListener(this);
//		radioItemApplication.setSelected(true);
		JRadioButtonMenuItem radioItemStandard = new JRadioButtonMenuItem(
				"Standard");
		radioItemStandard.setActionCommand("viewStd");
		radioItemStandard.addActionListener(this);
		if (this.opTree.getLevel().equals(ALDAOperator.Level.APPLICATION))
			radioItemApplication.setSelected(true);
		else
			radioItemStandard.setSelected(true);
		operatorLevelGroup.add(radioItemApplication);
		operatorLevelGroup.add(radioItemStandard);
		opSelectM.add(radioItemApplication);
		opSelectM.add(radioItemStandard);

		JMenu actionsM = new JMenu("Actions");
		JMenuItem runFlowItem = new JMenuItem("Run Workflow");
		runFlowItem.setActionCommand("runFlow");
		runFlowItem.addActionListener(this);
		actionsM.add(runFlowItem);

		// add to menu bar and finally to the main window
		mainWindowMenu.add(fileM);
		mainWindowMenu.add(opSelectM);
		mainWindowMenu.add(actionsM);
		mainWindow.setJMenuBar(mainWindowMenu);

		// set up main window
		mainWindow.pack();
		mainWindow.setVisible(true);
	}

	/**
	 * Configure collection of operators belonging to the standard set.
	 * @return	List of standard operators.
	 */
	protected Collection<String> configureCollectionStandardOps() {
		return ALDClassInfo.lookupOperators(ALDAOperator.Level.STANDARD,
				ALDAOperator.ExecutionMode.SWING);
	}
	
	/**
	 * Configure collection of operators belonging to the application set.
	 * @return	List of application operators.
	 */
	protected Collection<String> configureCollectionApplicationOps() {
		return ALDClassInfo.lookupOperators(ALDAOperator.Level.APPLICATION,
				ALDAOperator.ExecutionMode.SWING);
	}
	
	/**
	 * Show file dialog to save or load a work flow.
	 * 
	 * @param save
	 *            True if work flow has to be saved.
	 * @return  File to save to or load from the work flow.
	 */
	private File showDialog(boolean save) {
		File file = null;
		/* TODO
		 * Locale loc = Locale.ENGLISH; JFileChooser.setDefaultLocale(loc);
		 */
		JFileChooser fileDialog = new JFileChooser();

		if (save) {
//			fileDialog.setDialogType(JFileChooser.SAVE_DIALOG);
			fileDialog.setDialogTitle("Save workspace...");
			fileDialog.setApproveButtonText("Save");
			fileDialog.updateUI();
		}
		else {
			fileDialog.setDialogType(JFileChooser.OPEN_DIALOG);
			fileDialog.setDialogTitle("Load workspace...");
			fileDialog.updateUI();
		}

		fileDialog.setCurrentDirectory(new File(System.getProperty("user.home")	+ "/Desktop"));
		fileDialog.setFileSelectionMode(JFileChooser.FILES_ONLY);

		if (fileDialog.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
			file = fileDialog.getSelectedFile();
		}
		return file;
	}

	/** Save the work flow into two files. The first one holds the structure of the graph
	 * which is written to <code>file</code>. The second file holds the configuration of
	 * all operators. Its name is constructed appending the extension <code>opsXMLFileExtension</code>
	 * to the name of <code>file</code>.
	 * 
	 * @param file To save the work flow to
	 */
	private void saveWorkflow(File file) {
		mxCodec enc = new mxCodec();
		mxCodecRegistry.addPackage("de.unihalle.informatik.Alida.gui.editor.ALDPort");
		mxCodecRegistry.register(new mxObjectCodec(new ALDPort()));
		try {
			mxUtils.writeFile( mxUtils.getXml(enc.encode(opGraph.getModel())),
					file.getAbsolutePath());
			this.mainWindow.setTitle("ALDEditorFrame: " + file.getAbsolutePath());
		} catch (IOException e1) {
			// TODO: currently blocks !!
			/*JOptionPane.showMessageDialog(null, "Can not write work flow to " + file.getAbsolutePath(),
					"Error", JOptionPane.OK_CANCEL_OPTION);
			 */
			e1.printStackTrace();
		}
		
		// save all operator configurations
		HashMap<String,ALDOperator> operatorMap = new HashMap<String,ALDOperator>();
		for (mxCell cell : nodeMap.values()) {
			ALDOperator op;
			try {
				op = opDescrMap.get(cell).getOperatorControlFrame().getOperator();
				operatorMap.put( cell.getValue().toString(), op);
			} catch (ALDOperatorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		PrintStream out;
		try {
			out = new PrintStream(file.getAbsolutePath() + "." + opsXMLFileExtension);
			XStream xstream = new XStream(new DomDriver());
			String xml = xstream.toXML(operatorMap);
			out.println(xml);
		} catch (FileNotFoundException e) {
			// TODO: currently blocks !!
			JOptionPane.showMessageDialog(null, "Can not write operator configurations to " + 
					file.getAbsolutePath() + "." + opsXMLFileExtension,
					"Error", JOptionPane.OK_CANCEL_OPTION);
			 
			e.printStackTrace();
		}
	}

	/** Load the work flow from two files. The first one holds the structure of the graph
	 * which is contained in <code>file</code>. The second file holds the configuration of
	 * all operators. Its name is constructed appending the extension <code>opsXMLFileExtension</code>
	 * to the name of <code>file</code>.
	 * 
	 * @param file To load the work flow from
	 */
	@SuppressWarnings("unchecked")
	private void loadWorkflow(File file) {
		this.workFlowLoaded = false;
		
		// first the work flow
		Document document;
		try {
			mxCodecRegistry.addPackage("de.unihalle.informatik.Alida.gui.editor.ALDPort");
			mxCodecRegistry.register(new mxObjectCodec(new ALDPort()));
			document = mxUtils.parseXml(mxUtils.readFile(file.getAbsolutePath()));
			mxCodec codec = new mxCodec(document);
			codec.decode(document.getDocumentElement(), opGraph.getModel());
			this.mainWindow.setTitle("ALDEditorFrame: "	+ file.getAbsolutePath());

			nodeMap.clear();
			opDescrMap.clear();
		} catch (IOException e1) {
			JOptionPane.showMessageDialog(null, "Could not load work flow from file " + 
					file.getAbsolutePath(),
			"Error", JOptionPane.OK_CANCEL_OPTION);
			return;
		} catch (Exception e1) {
			JOptionPane.showMessageDialog(null, "Could not load work flow from file " + 
					file.getAbsolutePath(),
			"Error", JOptionPane.OK_CANCEL_OPTION);
			return;
		}
		
		// de serialize all operator configurations from file
		FileInputStream in;
		try {
			in = new FileInputStream( file.getAbsolutePath() + "." + opsXMLFileExtension);
			XStream xstream = new XStream(new DomDriver());
			HashMap<String,ALDOperator> operatorMap= new HashMap<String,ALDOperator>();
			boolean loadindConfigOk = true;
			try {
				operatorMap = 	(HashMap<String,ALDOperator>)(xstream.fromXML(in));
			} catch (Exception e1) {
				loadindConfigOk = false;
				JOptionPane.showMessageDialog(null, "Could not load operator configurations from file " + 
						file.getAbsolutePath() + "." + opsXMLFileExtension,
						"Error", JOptionPane.OK_CANCEL_OPTION);
				e1.printStackTrace();
			}
			
			// set all operator configurations loaded and also register edges
			LinkedList<String> missingOperatorConfigurations = new LinkedList<String>();
			for (Object child : opGraph.getChildCells(opGraph.getDefaultParent())) {
				mxCell cell = (mxCell) child;
				if (cell.isVertex()) {
					String name = cell.getValue().toString();
					if ( operatorMap.get(name) != null ) {
						registerNode(cell, operatorMap.get(name));
					} else {
						// add newly instantiated (not configured) operator

						if ( loadindConfigOk ) {
							missingOperatorConfigurations.add( name);
						}
						String opName = cell.getId().split("@")[0];

						try {
							registerNode(cell,(ALDOperator) (Class.forName(opName).newInstance()));
						} catch (Exception e) {
							// may be show options pane
							JOptionPane.showMessageDialog(null, "Could not instantiate operator " + 
									opName,
									"Warning", JOptionPane.OK_CANCEL_OPTION);
						}
					}

				} else if (cell.isEdge()) {
					registerEdge(cell);
				}
			}
						
			if ( ! missingOperatorConfigurations.isEmpty() ) {
				String missingNames = new String();
				for (String opName : missingOperatorConfigurations) {
					missingNames += "<br>" +
					 opName;
				}
				JOptionPane.showMessageDialog(null, "<html>Could not load configurations for operators " +
						missingNames + "</html>",
						"Warning", JOptionPane.OK_CANCEL_OPTION);
			}
			
			this.workFlowLoaded = true;
			// validate work flow to color nodes correctly
			LinkedList<mxCell> allNodes = 
					this.opGraph.getAllNodesTopologicallyOrdered();
			LinkedList<mxCell> processed = new LinkedList<mxCell>();
			for (mxCell node : allNodes) {
				if (!processed.contains(node)) {
					processed.add(node);
					boolean readyToRun = this.opDescrMap.get(node).
									getOperatorControlFrame().validateParameters(false, false);
					if (readyToRun)
						this.setToConfigured(node.getValue().toString(), true);
					else {
						LinkedList<mxCell> successors = 
								this.opGraph.getSubsequentNodes(node);
						for (mxCell sNode : successors) {
							this.setToConfigured(sNode.getValue().toString(), false);
							processed.add(sNode);
						}
					}
				}
			}
		} catch (FileNotFoundException e) {
			this.actionPerformed(new ActionEvent(this,1,"new"));
			JOptionPane.showMessageDialog(null, 
					"Cannot load operator configurations from " + 
							file.getAbsolutePath() + "." + opsXMLFileExtension 
							+": file not found!", "Error", JOptionPane.ERROR_MESSAGE);
		} catch (ALDOperatorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();

		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		/* handle tree view options */
		if (command.equals("viewApps")) {
//			mainWindow.remove(opTreePane);
			this.opTree.setLevel(ALDAOperator.Level.APPLICATION);
//			Tree = opTree.getTree();
//			Tree.addMouseListener(new TreeMouseAdapter(this));
//			opTreePane = new JScrollPane(Tree);
//			mainWindow.add(BorderLayout.WEST, opTreePane);
//			mainWindow.setVisible(false);
//			mainWindow.setVisible(true);
		} else if (command.equals("viewStd")) {
//			mainWindow.remove(opTreePane);
			this.opTree.setLevel(ALDAOperator.Level.STANDARD);
//			Tree = opTree.getTree();
//			Tree.addMouseListener(new TreeMouseAdapter(this));
//			opTreePane = new JScrollPane(Tree);
//			mainWindow.add(BorderLayout.WEST, opTreePane);
//			mainWindow.setVisible(false);
//			mainWindow.setVisible(true);
			
		}

		else if (command.equals("new")) {
			// TODO remove everything
			opDescrMap.clear();
			nodeMap.clear();
			opGraph.removeCells(opGraph.getChildVertices(opGraph
					.getDefaultParent()));
		}

		// save button has been pressed
		else if (command.equals("save")) {
			final File file = showDialog(true);
			if (file != null) {
//				try {
//					Thread tSave = new Thread() {
//						@Override
//            public void run() {
							saveWorkflow(file);
//						}
//					};
//					tSave.start();
//					tSave.join();
//					JOptionPane.showMessageDialog(null, "Save successful",
//							"Message", JOptionPane.OK_CANCEL_OPTION);
//				} catch (InterruptedException e1) {
//					// TODO Auto-generated catch block
//					e1.printStackTrace();
//				}
			}
		}

		// load button has been pressed
		else if (command.equals("load")) {
			final File file = showDialog(false);
			if (file != null) {
				// get position of editor frame on screen
//				Point pos = this.mainWindow.getLocationOnScreen();
//				// build-up status window
//				JLabel dummyLabel = new JLabel(" ",SwingConstants.CENTER);
//				dummyLabel.setSize(300, 30);
//				dummyLabel.setVerticalAlignment(SwingConstants.CENTER);
//				this.messageString = 
//						new JLabel("   Loading workflow, please wait...      ", 
//								SwingConstants.LEFT);
//				this.messageString.setSize(340, 50);
//				this.messageString.setVerticalAlignment(SwingConstants.CENTER);
//				this.okButton = new JButton("Ok");
//				this.okButton.setActionCommand("workflowLoaded");
//				this.okButton.addActionListener(this);
//				this.okButton.setEnabled(false);
//				JPanel buttonPanel = new JPanel();
//				buttonPanel.add(this.okButton);
//
//				this.loadMessageFrame = new JDialog();
//				this.loadMessageFrame.setLayout(new BorderLayout());
//				this.loadMessageFrame.add(dummyLabel,BorderLayout.NORTH);
//				this.loadMessageFrame.add(this.messageString,BorderLayout.CENTER);
//				this.loadMessageFrame.add(dummyLabel,BorderLayout.CENTER);
//				this.loadMessageFrame.add(buttonPanel, BorderLayout.SOUTH);
//				this.loadMessageFrame.setSize(350,100);
//				this.loadMessageFrame.setLocation(pos.x+300,pos.y+250);
//				this.loadMessageFrame.setAlwaysOnTop(true);
//				this.loadMessageFrame.setVisible(true);

//				Thread tLoad = new Thread() {
//        	@Override
//          public void run() {
        		loadWorkflow(file);
//        	}
//        };
//        tLoad.start();
//        try {
//          tLoad.join();
//          this.messageString.setText("   Loading workflow, please wait...done.  ");
//          this.messageString.updateUI();
//          this.okButton.setEnabled(true);
//        	JOptionPane.showMessageDialog(null, "Load successful",
//        			"Message", JOptionPane.OK_CANCEL_OPTION);
//        } catch (InterruptedException e1) {
//          // TODO Auto-generated catch block
//          e1.printStackTrace();
//        }
//        try {
//          Thread.currentThread().wait(1);
//        } catch (InterruptedException e1) {
//          // TODO Auto-generated catch block
//          e1.printStackTrace();
//        }
			}
		}
//		else if (command.equals("workflowLoaded")) {
//			this.loadMessageFrame.setVisible(false);
//			this.workFlowLoaded = true;
//		}
		else if (command.equals("runFlow")) {
			this.nodesToBeExecuted =	opGraph.getAllNodesTopologicallyOrdered();
			this.runNextNode();
		}
		// quit button has been pressed
		else if (command.equals("quit")) {
			mainWindow.setVisible(false);
		}
	}

	
	/**
	 * Register a new node representing an operator.
	 * 
	 * @param cell
	 *            Node to be registered.
	 * @param file
	 *            File with operator configuration.
	 * 
	 * @throws ALDOperatorException
	 */
	private void registerNode(mxCell cell, ALDOperator op)
			throws ALDOperatorException {
		ALDOperatorDescriptor opDescr = new ALDOperatorDescriptor(
				new ALDOperatorControlFrame(op));
		

		nodeMap.put(cell.getValue().toString(), cell);
		opDescrMap.put(cell, opDescr);

		if (cell.getStyle().contains("strokeColor=green"))
			cell.setStyle(cell.getStyle().replace("strokeColor=green",
					"strokeColor=yellow"));

		for (int i = 0; i < cell.getChildCount(); i++) {
			mxCell port = (mxCell) cell.getChildAt(i);
			opDescr.addParameter(((ALDPort) port.getValue()).getParName(), op
					.getParameterDescriptor(((ALDPort) port.getValue())
							.getParName()), port);
		}
		// Important: 
		// only set editor frame after everything else has been initialized!
		opDescr.getOperatorControlFrame().setEditorFrame(this,
				cell.getValue().toString());
	}


	/**
	 * Register a new edge representing a link.
	 * 
	 * @param edge
	 *            Edge to be registered.
	 */
	private void registerEdge(mxCell edge) {
		String linkSource = edge.getSource().getParent().getValue().toString()
				+ "@" + ((ALDPort) edge.getSource().getValue()).getParName();
		String linkTarget = edge.getTarget().getParent().getValue().toString()
				+ "@" + ((ALDPort) edge.getTarget().getValue()).getParName();
		ALDOperatorDescriptor opDescrTarget = opDescrMap.get(edge.getTarget()
				.getParent());
		ALDOpParameterDescriptor descr = opDescrTarget
				.getParameterDescriptor(((ALDPort) edge.getTarget().getValue())
						.getParName());
		descr.setLink(true);
		opDescrTarget.getOperatorControlFrame().setDescriptorLinked(linkTarget,true);
		opDescrTarget.getOperatorControlFrame().changePanel(linkTarget,
				linkSource);
		opDescrMap.get(edge.getSource().getParent()).addChild(
				edge.getTarget().getParent().getValue().toString());
	}

	/**
	 * Builds a node with the operator at opNamePath and the name opName.
	 * 
	 * @param opNamePath
	 *            Path of the operator.
	 * @param opName
	 *            Name of the operator.
	 * @param posX
	 *            X-Position of the new node.
	 * @param posY
	 *            Y-Position of the new node.
	 * @return Node which represents operator.
	 * 
	 * @throws ALDOperatorException
	 */
	protected mxCell buildNode(String opNamePath, String opName, double posX,
			double posY) throws ALDOperatorException {
		if (nodeMap.containsKey(opName)) {
			int i = 2;
			while (nodeMap.containsKey(opName + "_" + i)) {
				i++;
			}
			opNamePath = opNamePath + "@" + i;
			opName = opName + "_" + i;
		}

//		ALDOperatorDescriptor opDescr = new ALDOperatorDescriptor(
//				new ALDOperatorControlFrame(opNamePath.split("@")[0]));
		ALDOperatorControlFrame opframe =
				this.getControlFrame(opNamePath.split("@")[0]);
		if (opframe == null) {
			throw new ALDOperatorException(OperatorExceptionType.INSTANTIATION_ERROR,
				"[ALDEditorFrame::buildNode()] control frame for operator is null!?");
		}
		ALDOperatorDescriptor opDescr = new ALDOperatorDescriptor(opframe);
//		opDescr.getOperatorControlFrame().setEditorFrame(this, opName);
		if (opDescr == null || opDescr.getOperatorControlFrame() == null) {
			throw new ALDOperatorException(OperatorExceptionType.INSTANTIATION_ERROR,
				"[ALDEditorFrame::buildNode()] " + 
					" operator descriptor is null or somehow corrupted...!?");
		}
		ALDOperator op = opDescr.getOperatorControlFrame().getOperator();
		if (op == null)
			return null;

		int height = (op.getInNames(null).size() + op.getInOutNames(null)
				.size()) * 20;
		mxCell v = (mxCell) opGraph.insertVertex(opGraph.getDefaultParent(),
				opNamePath, opName, posX, posY, 160, height);
		v.setConnectable(false);
		nodeMap.put(opName, v);
		opDescrMap.put(v, opDescr);

		v.setStyle("verticalLabelPosition=top;verticalAlign=bottom;fontStyle=1;strokeWidth=3.0;strokeColor=red");

		double countIn = 1 / ((double) (op.getInNames(null).size() + op
				.getInOutNames(null).size()) + 1);
		int posIn = 1;
		double countOut = 1 / ((double) (op.getOutNames().size() + op
				.getInOutNames(null).size()) + 1);
		int posOut = 1;

		// build a port for each input parameter
		for (String s : op.getInNames(null)) {
			double x = countIn * posIn;
			mxGeometry geo = new mxGeometry(0, x, 10, 10);
			geo.setOffset(new mxPoint(-5, -5));
			geo.setRelative(true);
			ALDPort obj = new ALDPort(s);
			mxCell port = new mxCell(obj, geo, null);
			opDescr.addParameter(s, op.getParameterDescriptor(s), port);

			ALDOpParameterDescriptor descr = opDescr.getParameterDescriptor(s);

			// required parameters are blue and not required are yellow
			if (descr.isRequired()) {
				port.setStyle("labelPosition=right;align=left;shape=ellipse;perimter=ellipsePerimeter");
			} else if (!descr.getSupplemental().booleanValue()) {
				port.setStyle("labelPosition=right;align=left;fillColor=yellow;shape=ellipse;perimter=ellipsePerimeter");
			} else {
				port.setStyle("labelPosition=right;align=left;fillColor=red;shape=ellipse;perimter=ellipsePerimeter");
			}

			port.setVertex(true);
			opGraph.addCell(port, v);

			if (descr.getHandlingMode() != Parameter.ExpertMode.STANDARD)
				port.setVisible(false);

			posIn++;
		}

		// build a port for each output parameter
		for (String s : op.getOutNames()) {
			double x = countOut * posOut;
			mxGeometry geo = new mxGeometry(1, x, 10, 10);
			geo.setOffset(new mxPoint(-5, -5));
			geo.setRelative(true);
			ALDPort obj = new ALDPort(s);
			mxCell port = new mxCell(obj, geo,
					"labelPosition=left;align=right;shape=ellipse;perimter=ellipsePerimeter");
			opDescr.addParameter(s, op.getParameterDescriptor(s), port);

			port.setVertex(true);
			opGraph.addCell(port, v);

			posOut++;
		}

		// build a port for each inout parameter
		for (String s : op.getInOutNames(null)) {
			double x = countIn * posIn;
			mxGeometry geo1 = new mxGeometry(1, x, 10, 10);
			geo1.setOffset(new mxPoint(-5, -5));
			geo1.setRelative(true);
			ALDPort obj1 = new ALDPort(s);
			mxCell port1 = new mxCell(obj1, geo1,
					"labelPosition=left;align=right");

			mxGeometry geo2 = new mxGeometry(0, x, 10, 10);
			geo2.setOffset(new mxPoint(-5, -5));
			geo2.setRelative(true);
			ALDPort obj2 = new ALDPort(s);
			mxCell port2 = new mxCell(obj2, geo2,
					"labelPosition=right;align=left");

			opDescr.addParameter(s, op.getParameterDescriptor(s), port1);
			opDescr.addParameter(s, op.getParameterDescriptor(s), port2);

			port1.setVertex(true);
			opGraph.addCell(port1, v);
			port2.setVertex(true);
			opGraph.addCell(port2, v);

			posIn++;
		}
		opDescr.getOperatorControlFrame().setEditorFrame(this, opName);

		opGraph.refresh();
		return v;
	}

	/**
	 * Generates an operator control frame for the selected operator.
	 * @param opName	Chosen operator.
	 * @return	Operator control frame.
	 */
	protected ALDOperatorControlFrame getControlFrame(String opName) {
  	ALDOperatorControlFrame frame = null;
		try {
			frame = new ALDOperatorControlFrame(opName);
		} catch (ALDOperatorException e) {
			Object[] okOption = { "OK" };
			JOptionPane.showOptionDialog(
					null,
					"Could not instante chosen operator \"" + name + "\"\n" +
				  "Something seems to be wrong with its class implementation...",
				  "Warning", JOptionPane.DEFAULT_OPTION,
				  JOptionPane.WARNING_MESSAGE, null, okOption, okOption[0]);
			frame = null;
		}
  	return frame;
	}
	
	public void runNextNode() {
		if (!this.nodesToBeExecuted.isEmpty()) {
			mxCell nextToRun = this.nodesToBeExecuted.pop();
			this.runOpNode(nextToRun.getValue().toString(), false);
		}
	}
	
	/**
	 * Run the node representing an operator.
	 * 
	 * @param node
	 *            Node to run.
	 * @param wait
	 *            True if operator has to wait until finished.
	 * @return True if operator run was successful.
	 */
	public boolean runOpNode(String node, boolean wait) {
		mxCell cell = nodeMap.get(node);
		if (cell.getStyle().contains("strokeColor=yellow")
				|| cell.getStyle().contains("strokeColor=green")) {
			ALDOperatorDescriptor opDescr = opDescrMap.get(cell);

			boolean showResult = false;
			if (opDescr.getChildList().isEmpty()||this.nodesToBeExecuted.isEmpty()) {
				opDescr.getOperatorControlFrame().setProviderInteractionLevel(
						ProviderInteractionLevel.ALL_ALLOWED);
				showResult = true;
			}
			else {
				opDescr.getOperatorControlFrame().setProviderInteractionLevel(
						ProviderInteractionLevel.ALL_FORBIDDEN);
				showResult = false;
			}

			try {
				if (opDescr.getOperatorControlFrame().runOperator(wait, showResult)) {
//					while (!opDescr.getOperatorControlFrame().threadStatus.equals(OpThreadStatus.THREAD_STOPPED)) {
//						try {
//	            Thread.sleep(10);
//            } catch (InterruptedException e) {
//	            // TODO Auto-generated catch block
//	            e.printStackTrace();
//            }
//					}
					return true;
				}
			} catch (ALDOperatorException e) {
				JOptionPane.showMessageDialog(null, "Running node \"" + node
						+ "\" failed!" + " Reason: \n" + e.getCommentString(),
						"Warning", JOptionPane.OK_CANCEL_OPTION);
			}
		} else {
			if (cell.getStyle().contains("strokeColor=red"))
				JOptionPane.showMessageDialog(null, "The node " + node
						+ " isn't configured", "Warning",
						JOptionPane.OK_CANCEL_OPTION);
		}
		return false;
	}

	/**
	 * Run a graph with operations.
	 * 
	 * @param start
	 *            Node to start with.
	 */
	public void runOpGraph(String start) {
		ALDOperatorDescriptor opDescr = opDescrMap.get(nodeMap.get(start));
		if (!runOnlyNode) {
			if (opDescr.getChildList().isEmpty())
				opDescr.getOperatorControlFrame().showResult();
			for (String s : opDescr.getChildList()) {
				runOpNode(s, false);
			}
		} else
			opDescr.getOperatorControlFrame().showResult();
	}

	// TODO
	public void setToConfigured(String node, boolean ready) {
		mxCell cell = nodeMap.get(node);
		Color formerColor = null;
		if (ready) {
			// node is properly configured, change color to yellow
			if (cell.getStyle() != null) {
				if (cell.getStyle().contains("strokeColor=red"))
					formerColor = Color.red;
				else
					formerColor = Color.green;
				cell.setStyle(cell.getStyle().replace("strokeColor=red",
						"strokeColor=yellow"));
				cell.setStyle(cell.getStyle().replace("strokeColor=green",
						"strokeColor=yellow"));
			}
			else {
				cell.setStyle("verticalLabelPosition=top;verticalAlign=bottom;" +
						"fontStyle=1;strokeWidth=3.0;strokeColor=yellow");
			}
			// change color of all subsequent nodes as well
			if (this.workFlowLoaded) {
				LinkedList<mxCell> successors = 
						this.opGraph.getSubsequentNodesTopologicallyOrdered(cell);
				for (mxCell c: successors) {
					if (formerColor != null && formerColor.equals(Color.green)) {
						c.setStyle(c.getStyle().replace("strokeColor=green",
								"strokeColor=yellow"));
					}
					else {
						boolean readyToRun = this.opDescrMap.get(c).
								getOperatorControlFrame().validateParameters(false, false);
						if (readyToRun) {
							c.setStyle(c.getStyle().replace("strokeColor=red",
									"strokeColor=yellow"));
						}
						else {
							break;
						}
					}
				}
			}
		} else {
			// node is not properly configured, change color to red
			if (cell.getStyle() != null) {
				if (cell.getStyle().contains("strokeColor=red"))
					formerColor = Color.red;
				else
					formerColor = Color.green;
				cell.setStyle(cell.getStyle().replace("strokeColor=yellow",
						"strokeColor=red"));
				cell.setStyle(cell.getStyle().replace("strokeColor=green",
						"strokeColor=red"));
			}
			else {
				cell.setStyle("verticalLabelPosition=top;verticalAlign=bottom;" +
						"fontStyle=1;strokeWidth=3.0;strokeColor=red");
			}
			// change color of all subsequent nodes as well
			if (this.workFlowLoaded) {
				LinkedList<mxCell> successors = 
						this.opGraph.getSubsequentNodesTopologicallyOrdered(cell);
				for (mxCell c: successors) {
					c.setStyle(c.getStyle().replace("strokeColor=green",
							"strokeColor=red"));
					c.setStyle(c.getStyle().replace("strokeColor=yellow",
							"strokeColor=red"));
				}
			}
			else {
				// validate subsequent nodes and change color accordingly
			}
		}
		opGraph.refresh();
	}

	public void setToInExecution(String node, boolean executed) {
		mxCell cell = nodeMap.get(node);
		if (executed)
			cell.setStyle(cell.getStyle().replace("strokeColor=blue",
					"strokeColor=green"));
		else
			cell.setStyle(cell.getStyle().replace("strokeColor=yellow",
					"strokeColor=blue"));
		opGraph.refresh();
	}

	/**
	 * Get the value of a given link.
	 * 
	 * @param link
	 *            Link with operator and parameter.
	 * @return Value of the given link.
	 */
	public Object getValue(String link) {
		String[] linkParts = link.split("@");
		ALDOperatorControlFrame frame = opDescrMap.get(
				nodeMap.get(linkParts[0])).getOperatorControlFrame();
		Object value = null;
		try {
			value = frame.getOperator().getParameter(linkParts[1]);
		} catch (ALDOperatorException e) {
			JOptionPane.showMessageDialog(null, "Getting value of \""
					+ linkParts[1] + " in " + linkParts[0] + "\" failed!"
					+ " Reason: \n" + e.getCommentString(), "Warning",
					JOptionPane.OK_CANCEL_OPTION);
		}
		return value;
	}

	/**
	 * Validate a edge between two ports.
	 * 
	 * @param linkSource
	 *            Source of the edge.
	 * @param linkTarget
	 *            Target of the edge.
	 * @return False if validation failed.
	 */
	public boolean validateLink(String linkSource, String linkTarget) {
		String[] sourceSplit = linkSource.split("@");
		String[] targetSplit = linkTarget.split("@");

		// Check if node exists
		if (!nodeMap.containsKey(sourceSplit[0])) {
			JOptionPane.showMessageDialog(null, "This node doesn't exist",
					"Warning", JOptionPane.OK_CANCEL_OPTION);
			return false;
		}

		ALDOperatorDescriptor opDescrSource = opDescrMap.get(nodeMap
				.get(sourceSplit[0]));
		ALDOperatorDescriptor opDescrTarget = opDescrMap.get(nodeMap
				.get(targetSplit[0]));

		// Check if parameter exists
		if (opDescrSource.getPort(sourceSplit[1]) == null) {
			JOptionPane.showMessageDialog(null, "This parameter doesn't exist",
					"Warning", JOptionPane.OK_CANCEL_OPTION);
			return false;
		}

		// Check if parameter can be linked depending on parameter direction
		if (opDescrSource.getParameterDescriptor(sourceSplit[1]).getDirection() == Direction.OUT
				&& (opDescrTarget.getParameterDescriptor(targetSplit[1])
						.getDirection() == Direction.IN || opDescrTarget
						.getParameterDescriptor(targetSplit[1]).getDirection() == Direction.INOUT)
				|| opDescrSource.getParameterDescriptor(sourceSplit[1])
						.getDirection() == Direction.INOUT
				&& (opDescrTarget.getParameterDescriptor(targetSplit[1])
						.getDirection() == Direction.IN || opDescrTarget
						.getParameterDescriptor(targetSplit[1]).getDirection() == Direction.INOUT)) {
			// Check if parameter can be linked depending on parameter class
			// if (opDescrSource.getParameterDescriptor(sourceSplit[1])
			// .getMyclass() == opDescrTarget.getParameterDescriptor(
			// targetSplit[1]).getMyclass()) {
			if (opDescrTarget
					.getParameterDescriptor(targetSplit[1])
					.getMyclass()
					.isAssignableFrom(
							opDescrSource
									.getParameterDescriptor(sourceSplit[1])
									.getMyclass())) {
				return true;
			}
			JOptionPane.showMessageDialog(null, "You can't connect from \""
      		+ opDescrSource.getParameterDescriptor(sourceSplit[1])
      				.getMyclass().getSimpleName()
      		+ "\" to \""
      		+ opDescrTarget.getParameterDescriptor(targetSplit[1])
      				.getMyclass().getSimpleName() + "\"!", "Warning",
      		JOptionPane.OK_CANCEL_OPTION);
		} else {
			JOptionPane.showMessageDialog(null, "You can't connect from \""
					+ opDescrSource.getParameterDescriptor(sourceSplit[1])
							.getDirection()
					+ "\" to \""
					+ opDescrTarget.getParameterDescriptor(targetSplit[1])
							.getDirection() + "\"!", "Warning",
					JOptionPane.OK_CANCEL_OPTION);
		}

		return false;
	}

	/**
	 * Draw an edge between two ports.
	 * 
	 * @param linkSource
	 *            Source of the edge.
	 * @param linkTarget
	 *            Target of the edge.
	 */
	public void drawEdge(String linkSource, String linkTarget) {
		ALDOperatorDescriptor opDescr = opDescrMap.get(nodeMap.get(linkSource
				.split("@")[0]));

		mxCell source = opDescr.getPort(linkSource.split("@")[1]);
		mxCell target = opDescrMap.get(nodeMap.get(linkTarget.split("@")[0]))
				.getPort(linkTarget.split("@")[1]);

		if (target.getEdgeCount() != 0) {
			mxCell edge = (mxCell) opGraph.getIncomingEdges(target)[0];
			String linkSource1 = edge.getSource().getParent().getValue()
					.toString()
					+ "@"
					+ ((ALDPort) edge.getSource().getValue()).getParName();
			String linkTarget1 = edge.getTarget().getParent().getValue()
					.toString()
					+ "@"
					+ ((ALDPort) edge.getTarget().getValue()).getParName();
			if (!(linkSource1.equals(linkSource) && linkTarget1
					.equals(linkTarget)))
				removeEdge(edge, true);
			else
				return;
		}

		opGraph.insertEdge(opGraph.getDefaultParent(), null, null, source,
				target);
		opDescr.addChild(linkTarget.split("@")[0]);
	}

	/**
	 * Remove an edge.
	 * 
	 * @param edge
	 *            Edge to be removed.
	 * @param change
	 *            True if link has changed and false if removed.
	 */
	private void removeEdge(mxCell edge, boolean change) {
		ALDOperatorDescriptor opDescr = opDescrMap.get(edge.getSource()
				.getParent());
//		ALDOperatorDescriptor opDescrTarget = opDescrMap.get(edge.getTarget()
//				.getParent());
		mxCell targetNode = (mxCell)(edge.getTarget().getParent());
		
		opDescr.removeChild(edge.getTarget().getParent().getValue().toString());

		ALDOpParameterDescriptor descr = opDescrMap.get(
				edge.getTarget().getParent()).getParameterDescriptor(
				((ALDPort) edge.getTarget().getValue()).getParName());
		String link = edge.getTarget().getParent().getValue().toString() + "@"
				+ ((ALDPort) edge.getTarget().getValue()).getParName();
		if (!change) {
			descr.setLink(false);
			opDescrMap.get(edge.getTarget().getParent()).getOperatorControlFrame()
			.setDescriptorLinked(link, false);
		}
		opDescrMap.get(edge.getTarget().getParent()).getOperatorControlFrame()
				.changePanel(link, null);
		if (!change) {
			// change node color
			if (ALDEditorFrame.this.opDescrMap.get(targetNode).getOperatorControlFrame().validateParameters(false, false)) {
				ALDEditorFrame.this.setToConfigured(targetNode.getValue().toString(),true);
			}
			else {
				ALDEditorFrame.this.setToConfigured(targetNode.getValue().toString(),false);
			}
		}

		opGraph.getModel().remove(edge);
	}

	/**
	 * MouseListener for the operator tree.
	 */
	protected class TreeMouseAdapter extends MouseAdapter {

		ActionListener listener;

		TreeMouseAdapter(ActionListener list) {
			this.listener = list;
		}

		@Override
		public void mouseClicked(MouseEvent e) {
			if (e.getClickCount() == 1) {
//				if (opTree.getOpNamePath() != null) {
//					tempOpNamePath = opTree.getOpNamePath();
//					tempOpName = opTree.getOpName();
//					opTreePane.setCursor(new Cursor(Cursor.HAND_CURSOR));
//					graphComponent.getGraphHandler().DEFAULT_CURSOR = new Cursor(
//							Cursor.HAND_CURSOR);
//				}
				if (ALDEditorFrame.this.opTree.getLastSelectedPathComponent() != null) {
					ALDEditorFrame.this.tempOpNamePath = 
							ALDEditorFrame.this.opTree.getLastSelectedPathComponent().getFullName();
					ALDEditorFrame.this.tempOpName = ALDEditorFrame.this.opTree.getLastSelectedPathComponent().toString();
					ALDEditorFrame.this.opTreePane.setCursor(new Cursor(Cursor.HAND_CURSOR));
					ALDEditorFrame.this.graphComponent.getGraphHandler().DEFAULT_CURSOR = 
							new Cursor(Cursor.HAND_CURSOR);
				}
			}

			// on double-click add selected operator to graph
			if (e.getClickCount() == 2) {
				// get selected node
        ALDOperatorChooserTreeNode node = 
        	(ALDOperatorChooserTreeNode)ALDEditorFrame.this.
        															opTree.getLastSelectedPathComponent();
				if (node != null && node.isOperator()) {
					try {
						mxCell tmpNode = ALDEditorFrame.this.buildNode(
								ALDEditorFrame.this.opTree.getLastSelectedPathComponent().
																																getFullName(), 
								ALDEditorFrame.this.opTree.getLastSelectedPathComponent().
																													toString(),	50, 50);
						if (tmpNode == null) {
							JOptionPane.showMessageDialog(null, "Building node \""
									+ ALDEditorFrame.this.opTree.getLastSelectedPathComponent() 
									+ "\" failed!", "Warning", JOptionPane.OK_CANCEL_OPTION);
							return;
						}
						ALDEditorFrame.this.tempOpNamePath = null;
						ALDEditorFrame.this.tempOpName = null;
						ALDEditorFrame.this.opTreePane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						ALDEditorFrame.this.graphComponent.getGraphHandler().DEFAULT_CURSOR = new Cursor(
								Cursor.DEFAULT_CURSOR);
					} catch (ALDOperatorException e1) {
						JOptionPane.showMessageDialog(null, "Building node \""
								+ ALDEditorFrame.this.opTree.getLastSelectedPathComponent() + "\" failed!"
								+ " Reason: \n" + e1.getCommentString(),
								"Warning", JOptionPane.OK_CANCEL_OPTION);
					}
				}
			}
		}
	}

	/**
	 * MouseListener for the graph with operators.
	 */
	class GraphMouseAdapter extends MouseAdapter {

		ActionListener listener;

		GraphMouseAdapter(ActionListener list) {
			this.listener = list;
		}

		@Override
		public void mousePressed(MouseEvent e) {
			
//			System.out.println("Mouse event caught...");
//			System.out.println(e.toString());
			
			// double-click on node opens control frame
			if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount()==2) {
				mxCell cell = (mxCell) ALDEditorFrame.this.graphComponent.getCellAt(
																														e.getX(),e.getY());
				// double-clicks on background are ignored
				if (cell != null) {
					if (cell.isVertex() && !cell.isConnectable()) {
						ALDOperatorControlFrame frame = 
							ALDEditorFrame.this.opDescrMap.get(cell).
																										getOperatorControlFrame();
						frame.setVisible(true);
					}
				}
			}
			// left-click on background initializes node for formerly selected op
			else if (e.getButton() == MouseEvent.BUTTON1 && tempOpName != null
					&& tempOpNamePath != null) {
				try {
					buildNode(tempOpNamePath, tempOpName, e.getX(), e.getY());
					tempOpNamePath = null;
					tempOpName = null;
//					opTreePane.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					graphComponent.getGraphHandler().DEFAULT_CURSOR = new Cursor(
							Cursor.DEFAULT_CURSOR);
				} catch (ALDOperatorException e1) {
					JOptionPane.showMessageDialog(null, "Building node \""
							+ opTree.getLastSelectedPathComponent() + "\" failed!" + " Reason: \n"
							+ e1.getCommentString(), "Warning",
							JOptionPane.OK_CANCEL_OPTION);
				}
			}
//			else if (e.getButton() == MouseEvent.BUTTON1 
//					&& (tempOpName == null || tempOpNamePath == null)) {
//				System.out.println("Got left click, edge draw...");
//				mxCell cell = (mxCell) ALDEditorFrame.this.graphComponent.getCellAt(
//						e.getX(),e.getY());
//				System.out.println(graphComponent.getGraphHandler().isMoveEnabled());
//				if (cell == null)
//					return;
//				if (cell.isEdge()) {
//					System.out.println("... related to an edge");
//					String linkSource = cell.getSource().getParent().getValue()
//							.toString()
//							+ "@"
//							+ ((ALDPort) cell.getSource().getValue()).getParName();
//					String linkTarget = cell.getTarget().getParent().getValue()
//							.toString()
//							+ "@"
//							+ ((ALDPort) cell.getTarget().getValue()).getParName();
//					mxCell targetNode = (mxCell)cell.getTarget().getParent();
//					System.out.println(linkSource);
//					System.out.println(linkTarget);
//				}
//			}
			// on right-click open context menu
			else if (e.getButton() == MouseEvent.BUTTON3) {
				mxCell cell = (mxCell) graphComponent.getCellAt(e.getX(),
						e.getY());
				if (cell != null) {
					if (cell.isVertex() && !cell.isConnectable()
							|| cell.isEdge()) {
						PopUpMenu menu = new PopUpMenu(e.getX(), e.getY());
						menu.show(e.getComponent(), e.getX(), e.getY());
					} else {
						// formerly the following code was for switching ports to text mode
//						ALDOperatorDescriptor opDescr = opDescrMap.get(cell
//								.getParent());
//						ALDOpParameterDescriptor descr = opDescr
//								.getParameterDescriptor(((ALDPort) cell
//										.getValue()).getParName());
//						String link = cell.getParent().getValue().toString()
//								+ "@"
//								+ ((ALDPort) cell.getValue()).getParName();
//						if (descr.getDirection() == Direction.IN
//								|| descr.getDirection() == Direction.INOUT) {
//							if (descr.isLink())
//								descr.setLink(false);
//							else
//								descr.setLink(true);
//							opDescr.getOperatorControlFrame().changePanel(link,
//									null);
//							opDescr.getOperatorControlFrame().setVisible(true);
//						} else {
//							JOptionPane.showMessageDialog(null,
//									"This parameter is an output!", "Warning",
//									JOptionPane.OK_CANCEL_OPTION);
//						}
					}
				}
				else {
					PopUpMenuBench menu = new PopUpMenuBench(e.getX(), e.getY());
					menu.show(ALDEditorFrame.this.graphComponent, e.getX(), e.getY());
				}
			}
		}
	}

	/**
	 * ConnectionListener for drawing an edge.
	 */
	class EdgeConnectListener implements mxIEventListener {
		
		private HashMap<mxCell, mxICell> edgeSources= new HashMap<mxCell, mxICell>();
		
		private HashMap<mxCell, mxICell> edgeTargets= new HashMap<mxCell, mxICell>();

		@Override
    public void invoke(Object sender, mxEventObject event) {
			
//			System.out.println("Received an event...");
//			System.out.println(event.getName());
			
			String eventName = event.getName();
			mxCell cell = null;
			if (eventName.equals("connect"))
				cell = (mxCell) event.getProperty("cell");
			else if (eventName.equals("connectCell"))
				cell = (mxCell) event.getProperty("edge");
			if (cell == null || !cell.isEdge()) {
				// nothing to do, if event is not related to an edge...
//				System.out.println("Event on vertex?!");
				return;
			}
			ALDOperatorDescriptor opDescrTarget = opDescrMap.get(cell
					.getTarget().getParent());
			ALDOpParameterDescriptor descr = opDescrTarget
					.getParameterDescriptor(((ALDPort) cell.getTarget()
							.getValue()).getParName());
			
			// if port is already linked to another input, abort operation
			if (descr.isLink()) {
//				System.out.println("Port is already linked...");
				mxICell target = this.edgeTargets.get(cell);
				if (target != null) {
					ALDOperatorDescriptor tdescr = opDescrMap.get(target.getParent());
					ALDOpParameterDescriptor tpdescr = tdescr
							.getParameterDescriptor(((ALDPort) target.getValue()).getParName());
					tpdescr.setLink(false);
					// TODO HACK!!!! Eigentlich sollte dieses explizite Linkumschalten
					//               unnötig sein... irgendwo werden Referenzen ausgetauscht?!
					tdescr.getOperatorControlFrame().setDescriptorLinked(target.getParent().getValue().toString() + "@" + ((ALDPort)target.getValue()).getParName(),false);
					tdescr.getOperatorControlFrame().changePanel(target.getParent().getValue().toString() + "@" + ((ALDPort)target.getValue()).getParName(),null);
				}
				opGraph.getModel().remove(cell);
				this.edgeSources.remove(cell);
				this.edgeTargets.remove(cell);
				return;
			}
			
			// if workflow graph is cyclic, abort operation
			if (ALDEditorFrame.this.opGraph.getAllNodesTopologicallyOrdered()==null){
//				System.out.println("Cyclic graph created...");
//				System.out.println("cell: " + cell.toString() + " , source = " + cell.getSource().toString() + " , target = " + cell.getTarget());
				opGraph.getModel().remove(cell);
				this.edgeSources.remove(cell);
				this.edgeTargets.remove(cell);
				return;
			}

			String linkSource = cell.getSource().getParent().getValue()
					.toString()
					+ "@"
					+ ((ALDPort) cell.getSource().getValue()).getParName();
			String linkTarget = cell.getTarget().getParent().getValue()
					.toString()
					+ "@"
					+ ((ALDPort) cell.getTarget().getValue()).getParName();
			mxCell targetNode = (mxCell)cell.getTarget().getParent();
			if (validateLink(linkSource, linkTarget)) {
//				System.out.println("Validate successful...");
				descr.setLink(true);
				opDescrTarget.getOperatorControlFrame().setDescriptorLinked(linkTarget,true);
				opDescrTarget.getOperatorControlFrame().changePanel(linkTarget,
						linkSource);
				opDescrMap.get(cell.getSource().getParent()).addChild(
						cell.getTarget().getParent().getValue().toString());
				// modify old target operator port
				mxICell target = null;
				if ((target = this.edgeTargets.get(cell)) != null) {
					ALDOperatorDescriptor tdescr = opDescrMap.get(target.getParent());
					ALDOpParameterDescriptor tpdescr = tdescr
							.getParameterDescriptor(((ALDPort) target.getValue()).getParName());
					tpdescr.setLink(false);
					// TODO HACK!!!! Eigentlich sollte dieses explizite Linkumschalten
					//               unnötig sein... irgendwo werden Referenzen ausgetauscht?!
					tdescr.getOperatorControlFrame().setDescriptorLinked(target.getParent().getValue().toString() + "@" + ((ALDPort)target.getValue()).getParName(),false);
					tdescr.getOperatorControlFrame().changePanel(target.getParent().getValue().toString() + "@" + ((ALDPort)target.getValue()).getParName(),null);
				}
				// memorize edge
				this.edgeSources.put(cell, cell.getSource());
				this.edgeTargets.put(cell, cell.getTarget());
//				System.out.println("cell: " + cell.toString() + " , source = " + cell.getSource().toString() + " , target = " + cell.getTarget());
			}
			else {
//				System.out.println("Validate failed...");
				if (this.edgeTargets.get(cell) == null) {
//					System.out.println("cell: " + cell.toString() + " , source = " + cell.getSource().toString() + " , target = " + cell.getTarget());
//					System.out.println("Edge was not linked before...");
					opGraph.getModel().remove(cell);
					this.edgeSources.remove(cell);
					this.edgeTargets.remove(cell);
				}
				else {
					mxICell target = this.edgeTargets.get(cell);
					ALDOperatorDescriptor tdescr = opDescrMap.get(target.getParent());
					ALDOpParameterDescriptor tpdescr = tdescr
							.getParameterDescriptor(((ALDPort) target.getValue()).getParName());
					tpdescr.setLink(false);
					// TODO HACK!!!! Eigentlich sollte dieses explizite Linkumschalten
					//               unnötig sein... irgendwo werden Referenzen ausgetauscht?!
					tdescr.getOperatorControlFrame().setDescriptorLinked(target.getParent().getValue().toString() + "@" + ((ALDPort)target.getValue()).getParName(),false);
					tdescr.getOperatorControlFrame().changePanel(target.getParent().getValue().toString() + "@" + ((ALDPort)target.getValue()).getParName(),null);
//					System.out.println("Edge was linked before...");
//					System.out.println("cell: " + cell.toString() + " , source = " + cell.getSource().toString() + " , target = " + cell.getTarget());
					opGraph.getModel().remove(cell);
					this.edgeSources.remove(cell);
					this.edgeTargets.remove(cell);
					//					cell.setTarget(this.edgeTargets.get(cell));
					//					opGraph.insertEdge(null, null, null, this.edgeSources.get(cell), this.edgeTargets.get(cell));
					//					System.out.println("cell: " + cell.toString() + " , source = " + cell.getSource().toString() + " , target = " + cell.getTarget());
				}
			}
			// change node color
			if (ALDEditorFrame.this.opDescrMap.get(targetNode).getOperatorControlFrame().validateParameters(false, false)) {
				ALDEditorFrame.this.setToConfigured(targetNode.getValue().toString(),true);
			}
			else {
				ALDEditorFrame.this.setToConfigured(targetNode.getValue().toString(),false);
			}
		}
	}

	/**
	 * Context Menu of nodes.
	 */
	protected class PopUpMenu extends JPopupMenu implements ActionListener {
		private mxCell cell;

		protected PopUpMenu(int x, int y) {
			cell = (mxCell) graphComponent.getCellAt(x, y);

			if (cell.isVertex()) {
				JMenuItem confItem = new JMenuItem("Configure operator");
				confItem.setActionCommand("conf");
				confItem.addActionListener(this);

				JMenu runMenu = new JMenu("Run");

				JMenuItem runFlowItem = new JMenuItem("Workflow");
				runFlowItem.setActionCommand("runFlow");
				runFlowItem.addActionListener(this);

//				JMenu runFromMenu = new JMenu("from here");
//				JMenuItem runItem1 = new JMenuItem("with backtracking");
//				runItem1.setActionCommand("runFromHere");
//				runItem1.addActionListener(this);
				JMenuItem runFromItem = new JMenuItem("...from here");
				runFromItem.setActionCommand("runFromHere");
				runFromItem.addActionListener(this);
//				JMenuItem runItem2 = new JMenuItem("without backtracking");
//				runItem2.setActionCommand("run2");
//				runItem2.addActionListener(this);
//				runFromMenu.add(runItem1);
//				runFromMenu.add(runItem2);

				JMenuItem runUptoItem = new JMenuItem("...up to here");
				runUptoItem.setActionCommand("runUptoHere");
				runUptoItem.addActionListener(this);
//				JMenu runNodeMenu = new JMenu("node");
//				JMenuItem runNodeItem1 = new JMenuItem("with backtracking");
//				runNodeItem1.setActionCommand("runNode1");
//				runNodeItem1.addActionListener(this);
//				JMenuItem runNodeItem2 = new JMenuItem("without backtracking");
//				runNodeItem2.setActionCommand("runNode2");
//				runNodeItem2.addActionListener(this);
//				runNodeMenu.add(runNodeItem1);
//				runNodeMenu.add(runNodeItem2);

				runMenu.add(runFlowItem);
				runMenu.add(runFromItem);
				runMenu.add(runUptoItem);

				JMenuItem showItem = new JMenuItem("Show Results");
				showItem.setActionCommand("show");
				showItem.addActionListener(this);
				if (!cell.getStyle().contains("strokeColor=green")) {
					showItem.setEnabled(false);
				}

//				JMenu copyNodeMenu = new JMenu("Copy node");
//				JMenuItem copyItem1 = new JMenuItem("without configuration");
//				copyItem1.setActionCommand("copyNode1");
//				copyItem1.addActionListener(this);
//				JMenuItem copyItem2 = new JMenuItem("with links");
//				copyItem2.setActionCommand("copyNode2");
//				copyItem2.addActionListener(this);
//				JMenuItem copyItem3 = new JMenuItem("with configuration");
//				copyItem3.setActionCommand("copyNode3");
//				copyItem3.addActionListener(this);
//				copyNodeMenu.add(copyItem1);
//				copyNodeMenu.add(copyItem2);
//				copyNodeMenu.add(copyItem3);

				JMenuItem removeItem = new JMenuItem("Remove node");
				removeItem.setActionCommand("removeNode");
				removeItem.addActionListener(this);

				JMenu viewMenu = new JMenu("Viewmode");
				JMenuItem standardItem = new JMenuItem("Standard");
				standardItem.setActionCommand("standard");
				standardItem.addActionListener(this);
				JMenuItem advancedItem = new JMenuItem("Advanced");
				advancedItem.setActionCommand("advanced");
				advancedItem.addActionListener(this);
				viewMenu.add(standardItem);
				viewMenu.add(advancedItem);

				this.add(confItem);
				this.add(runMenu);
				this.add(showItem);
				this.addSeparator();
				this.add(viewMenu);
//				this.add(copyNodeMenu);
				this.add(removeItem);
			} else {
				JMenuItem removeItem = new JMenuItem("Remove edge");
				removeItem.setActionCommand("removeEdge");
				removeItem.addActionListener(this);
				this.add(removeItem);
			}
		}

		/**
		 * Change the mode for displaying parameters.
		 * 
		 * @param cell
		 *            Node with the operator.
		 * @param handlingMode
		 *            New handling mode.
		 */
		private void changeView(mxCell cell, Parameter.ExpertMode handlingMode) {
			for (int i = 0; i < cell.getChildCount(); i++) {
				ALDOpParameterDescriptor descr = opDescrMap.get(cell)
						.getParameterDescriptor(
								((ALDPort) cell.getChildAt(i).getValue())
										.getParName());
				if (descr.getDirection() == Direction.IN
						|| descr.getDirection() == Direction.INOUT) {
					if (handlingMode == Parameter.ExpertMode.STANDARD) {
						if (descr.getHandlingMode() == handlingMode) {
							cell.getChildAt(i).setVisible(true);
						} else {
							cell.getChildAt(i).setVisible(false);
						}
					} else {
						cell.getChildAt(i).setVisible(true);
					}
				}
			}
			opGraph.refresh();
			opDescrMap.get(cell).getOperatorControlFrame()
					.changeMode(handlingMode);
		}

		@Override
    public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();

			if (command.equals("conf")) {
				ALDOperatorControlFrame frame = opDescrMap.get(cell)
						.getOperatorControlFrame();
				frame.setVisible(true);
			} else if (command.equals("runFlow")) {
//				runOnlyNode = false;
//				opDescrMap.get(cell).setReverse(true);
//				for (mxCell cell : nodeMap.values()) {
//					int count = 0;
//					for (int i = 0; i < cell.getChildCount(); i++) {
//						Object[] inEdges = opGraph.getIncomingEdges(cell
//								.getChildAt(i));
//						count += inEdges.length;
//					}
//					if (count == 0)
//						runOpNode(cell.getValue().toString(), false);
//				}
				ALDEditorFrame.this.nodesToBeExecuted = 
						opGraph.getAllNodesTopologicallyOrdered();
				ALDEditorFrame.this.runNextNode();
//				for (mxCell c : nodesToRun) {
//					System.out.println("Node: " + 
//							opDescrMap.get(c).getOperatorControlFrame().getALDOperator().getName());
//				}
//				for (mxCell c: nodesToRun) {
//					System.out.println("Running " + opDescrMap.get(c).getOperatorControlFrame().getALDOperator().getName());
//					runOpNode(cell.getValue().toString(), true);
//					try {
//	          Thread.sleep(1000);
//          } catch (InterruptedException e1) {
//	          // TODO Auto-generated catch block
//	          e1.printStackTrace();
//          }
//				}
			} else if (command.equals("runFromHere")) {
				// with backtracking
				ALDEditorFrame.this.nodesToBeExecuted = 
						opGraph.getSubsequentSubgraphNodesTopologicallyOrdered(cell);
				ALDEditorFrame.this.runNextNode();
//				runOnlyNode = false;
//				opDescrMap.get(cell).setReverse(true);
//				runOpNode(cell.getValue().toString(), false);
//			} else if (command.equals("run2")) {
//				// without backtracking
//				runOnlyNode = false;
//				opDescrMap.get(cell).setReverse(false);
//				runOpNode(cell.getValue().toString(), false);
//			} else if (command.equals("runNode1")) {
//				// with backtracking
//				runOnlyNode = true;
//				opDescrMap.get(cell).setReverse(true);
//				runOpNode(cell.getValue().toString(), false);
//			} else if (command.equals("runNode2")) {
//				// without backtracking
//				runOnlyNode = true;
//				opDescrMap.get(cell).setReverse(false);
//				runOpNode(cell.getValue().toString(), false);
			} else if (command.equals("runUptoHere")) {
				// with backtracking
				ALDEditorFrame.this.nodesToBeExecuted = 
						opGraph.getPrecedingNodesTopologicallyOrdered(cell);
//				for (mxCell c: ALDEditorFrame.this.nodesToBeExecuted) {
//					try {
//	          System.out.println("Running " + 
//opDescrMap.get(c).getOperatorControlFrame().getOperator().getName());
//          } catch (ALDOperatorException e1) {
//	          // TODO Auto-generated catch block
//	          e1.printStackTrace();
//          }
//				}
				ALDEditorFrame.this.runNextNode();
			} else if (command.equals("show")) {
				ALDOperatorControlFrame frame = opDescrMap.get(cell)
						.getOperatorControlFrame();
				frame.showResult();
			} else if (command.equals("standard")) {
				changeView(cell, Parameter.ExpertMode.STANDARD);
			} else if (command.equals("advanced")) {
				changeView(cell, Parameter.ExpertMode.ADVANCED);
			} else if (command.equals("copyNode1")) {
				try {
					buildNode(cell.getId().split("@")[0], cell.getValue()
							.toString().split("_")[0], cell.getGeometry()
							.getX() + 20, cell.getGeometry().getY() + 20);
				} catch (ALDOperatorException e1) {
					JOptionPane.showMessageDialog(null, "Copying node \""
							+ cell.getValue().toString() + "\" failed!"
							+ " Reason: \n" + e1.getCommentString(), "Warning",
							JOptionPane.OK_CANCEL_OPTION);
				}
			} else if (command.equals("copyNode2")) {
				try {
					mxCell copy = buildNode(cell.getId().split("@")[0], cell
							.getValue().toString().split("_")[0], cell
							.getGeometry().getX() + 20, cell.getGeometry()
							.getY() + 20);
					for (int i = 0; i < cell.getChildCount(); i++) {
						Object[] inEdges = opGraph.getIncomingEdges(cell
								.getChildAt(i));
						if (inEdges.length != 0) {
							ALDOperatorDescriptor opDescrTarget = opDescrMap
									.get(copy);
							ALDOpParameterDescriptor descr = opDescrTarget
									.getParameterDescriptor(((ALDPort) copy
											.getChildAt(i).getValue())
											.getParName());
							String linkSource = ((mxCell) inEdges[0])
									.getSource().getParent().getValue()
									.toString()
									+ "@"
									+ ((ALDPort) ((mxCell) inEdges[0])
											.getSource().getValue())
											.getParName();
							String linkTarget = copy.getValue().toString()
									+ "@"
									+ ((ALDPort) cell.getChildAt(i).getValue())
											.getParName();
							descr.setLink(true);
							opDescrTarget.getOperatorControlFrame()
									.changePanel(linkTarget, linkSource);
							opDescrTarget.getOperatorControlFrame().setDescriptorLinked(linkTarget,true);
							drawEdge(linkSource, linkTarget);
						}
					}
					opGraph.refresh();
				} catch (ALDOperatorException e1) {
					JOptionPane.showMessageDialog(null, "Copying node \""
							+ cell.getValue().toString() + "\" failed!"
							+ " Reason: \n" + e1.getCommentString(), "Warning",
							JOptionPane.OK_CANCEL_OPTION);
				}
			} else if (command.equals("copyNode3")) {
				try {
					mxCell copy = buildNode(cell.getId().split("@")[0], cell
							.getValue().toString().split("_")[0], cell
							.getGeometry().getX() + 20, cell.getGeometry()
							.getY() + 20);

					ALDOperatorDescriptor opDescr1 = opDescrMap.get(cell);
					ALDOperatorDescriptor opDescr2 = opDescrMap.get(copy);

					opDescr1.getOperatorControlFrame().copyConfiguration(
							opDescr2.getOperatorControlFrame());

					for (int i = 0; i < cell.getChildCount(); i++) {
						Object[] inEdges = opGraph.getIncomingEdges(cell
								.getChildAt(i));
						if (inEdges.length != 0) {
							ALDOperatorDescriptor opDescrTarget = opDescrMap
									.get(copy);
							ALDOpParameterDescriptor descr = opDescrTarget
									.getParameterDescriptor(((ALDPort) copy
											.getChildAt(i).getValue())
											.getParName());
							String linkSource = ((mxCell) inEdges[0])
									.getSource().getParent().getValue()
									.toString()
									+ "@"
									+ ((ALDPort) ((mxCell) inEdges[0])
											.getSource().getValue())
											.getParName();
							String linkTarget = copy.getValue().toString()
									+ "@"
									+ ((ALDPort) cell.getChildAt(i).getValue())
											.getParName();
							descr.setLink(true);
							opDescrTarget.getOperatorControlFrame().setDescriptorLinked(linkTarget,true);
							opDescrTarget.getOperatorControlFrame()
									.changePanel(linkTarget, linkSource);
							drawEdge(linkSource, linkTarget);
						}
					}
					opGraph.refresh();
				} catch (ALDOperatorException e1) {
					JOptionPane.showMessageDialog(null, "Copying node \""
							+ cell.getValue().toString() + "\" failed!"
							+ " Reason: \n" + e1.getCommentString(), "Warning",
							JOptionPane.OK_CANCEL_OPTION);
				} catch (ALDDataIOException e2) {
					// TODO Auto-generated catch block
					e2.printStackTrace();
				}
			} else if (command.equals("removeNode")) {
				for (int i = 0; i < cell.getChildCount(); i++) {
					Object[] inEdges = opGraph.getIncomingEdges(cell
							.getChildAt(i));

					if (inEdges.length != 0) {
						removeEdge((mxCell) inEdges[0], false);
					}

					Object[] outEdges = opGraph.getOutgoingEdges(cell
							.getChildAt(i));
					for (int j = 0; j < outEdges.length; j++) {
						removeEdge((mxCell) outEdges[j], false);
					}
				}
				opDescrMap.remove(cell);
				nodeMap.remove(cell.getValue());
				opGraph.getModel().remove(cell);
			} else if (command.equals("removeEdge")) {
				removeEdge(cell, false);
			}
		}
	}
	
	/**
	 * Context Menu of nodes.
	 */
	protected class PopUpMenuBench extends JPopupMenu implements ActionListener {
		private mxCell cell;

		protected PopUpMenuBench(int x, int y) {
			JMenuItem newItem = new JMenuItem("New Workflow");
			newItem.setActionCommand("newFlow");
			newItem.addActionListener(this);
			JMenuItem confItem = new JMenuItem("Run Workflow");
			confItem.setActionCommand("runFlow");
			confItem.addActionListener(this);
			this.add(newItem);
			this.addSeparator();
			this.add(confItem);
		}

		@Override
    public void actionPerformed(ActionEvent e) {
			String command = e.getActionCommand();
			if (command.equals("newFlow")) { 
				ALDEditorFrame.this.actionPerformed(new ActionEvent(this,1,"new"));
			} else if (command.equals("runFlow")) {
				ALDEditorFrame.this.actionPerformed(new ActionEvent(this,1,"runFlow"));
			}
		}
	}
	
//	protected class myConnectionHandler extends mxConnectionHandler {
//
//		public myConnectionHandler(mxGraphComponent arg0) {
//	    super(arg0);
//	    // TODO Auto-generated constructor stub
//    }

//		@Override
//		public void mousePressed(MouseEvent e) {
//			System.out.println("Mouse was pressed..");
//		}
//		
//		@Override
//		public void mouseReleased(MouseEvent e) {
//			System.out.println("Mouse was released...");
//		}
//
//		public void 	mouseDragged(MouseEvent e) {
//			System.out.println("Mouse was dragged...");			
//		}
//    
//		public void mouseMoved(MouseEvent e) {
//			System.out.println("Mouse was moved...");
//		}
//	}
	
//	protected class myGraphComponent extends mxGraphComponent {
//		public myGraphComponent(mxGraph arg0) {
//	    super(arg0);
//	    // TODO Auto-generated constructor stub
//    }
//
//		protected mxConnectionHandler createConnectionHandler() {
//			return new myConnectionHandler(this);
//		}
//	}
}
