package de.unihalle.informatik.Alida.gui.editor;

import java.util.*;

import com.mxgraph.model.mxCell;



import de.unihalle.informatik.Alida.annotations.Parameter;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.exceptions.ALDOperatorException.OperatorExceptionType;
import de.unihalle.informatik.Alida.gui.ALDOperatorControlFrame;
import de.unihalle.informatik.Alida.operator.ALDOpParameterDescriptor;
import de.unihalle.informatik.Alida.operator.ALDParameterDescriptor;

/**
 * This class contains the main informations to configure a node representing an
 * operator.
 * 
 * @author Steven Kirchner
 * 
 */


public class ALDOperatorDescriptor {

	/**
	 * Associated configuration/control frame.
	 */
	private ALDOperatorControlFrame controlFrame = null;

	/**
	 * List of the following operations.
	 */
	private ArrayList<String> childList = new ArrayList<String>();

	/**
	 * Maps a name of the parameter to its ALDParameterDescriptor.
	 */
	private HashMap<String, ALDParameterDescriptor> parameterMap = new HashMap<String, ALDParameterDescriptor>();

	/**
	 * Maps a name of the parameter to its cell.
	 */
	private HashMap<String, mxCell> portMap = new HashMap<String, mxCell>();

	/**
	 * Returns the associated configuration frame.
	 * 
	 * @return associated configuration frame.
	 */
	public ALDOperatorControlFrame getOperatorControlFrame() {
		return this.controlFrame;
	}

	/**
	 * Returns the list of children.
	 * 
	 * @return List of following operations.
	 */
	public ArrayList<String> getChildList() {
		return this.childList;
	}

	/**
	 * Returns the parameter descriptor associated with a name.
	 * 
	 * @param parName
	 *            Name of the parameter.
	 * @return associated parameter descriptor.
	 */
	public ALDOpParameterDescriptor getParameterDescriptor(String parName) {
		return (ALDOpParameterDescriptor) this.parameterMap.get(parName);
	}

	/**
	 * Returns the cell associated with a name.
	 * 
	 * @param parName
	 *            Name of the parameter.
	 * @return associated cell.
	 */
	public mxCell getPort(String parName) {
		return this.portMap.get(parName);
	}

	/**
	 * Default constructor
	 * 
	 * @param _controlFrame
	 */
	public ALDOperatorDescriptor(ALDOperatorControlFrame _controlFrame) 
		throws ALDOperatorException {
		if (_controlFrame == null) {
			throw new ALDOperatorException(OperatorExceptionType.UNSPECIFIED_ERROR, 
					"[ALDOperatorDescriptor(...)] OperatorControlFrame is null!");
		}
		this.controlFrame = _controlFrame;
	}

	/**
	 * Add a new operation as child.
	 * 
	 * @param child
	 */
	public void addChild(String child) {
		this.childList.add(child);
	}

	/**
	 * Remove a operation as child.
	 * 
	 * @param child
	 */
	public void removeChild(String child) {
		this.childList.remove(child);
	}

	/**
	 * Maps the name of a parameter to its descriptor.
	 * 
	 * @param parName
	 *            Name of the parameter.
	 * @param parDes
	 *            Descriptor of the parameter.
	 */
	public void addParameter(String parName, ALDParameterDescriptor descr,
			mxCell port) {
		this.parameterMap.put(parName, descr);
		this.portMap.put(parName, port);
	}

	// TODO
	public void setReverse(Boolean reverse) {
		for (ALDParameterDescriptor descr : parameterMap.values()) {
			descr.setReverse(reverse);
		}
	}
}
