/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */
/* 
 * Most recent change(s):
 * 
 * $Rev: 6069 $
 * $Date: 2012-10-02 18:06:55 +0200 (Di, 02. Okt 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.core.gui;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;



import de.unihalle.informatik.Alida.exceptions.ALDOperatorException;
import de.unihalle.informatik.Alida.gui.ALDChooseOpNameFrame;
import de.unihalle.informatik.Alida.gui.ALDOperatorControlFrame;
import de.unihalle.informatik.Alida.gui.OnlineHelpDisplayer;

/**
 * Main window for selecting MiToBo annotated operators for running.
 * 
 * @author Stefan Posch
 * @author Birgit Moeller
 */

public class MTBChooseOpNameFrame extends ALDChooseOpNameFrame {

	/**
	 * Debug flag (not accessible from outside).
	 */
	protected boolean debug = false;
	
	/**
	 * Constructor.
	 */
	public MTBChooseOpNameFrame() {
		this.setTitle("MiToBo - OpRunner: simply choose an operator...");
		// load about icon
		String iconDataName = "/share/logo/MiToBo_logo.png";
		Image img = null;
		BufferedImage bi = null;
		Graphics g = null;
		InputStream is = null;
		try {
			ImageIcon icon;
			File iconDataFile = new File("./" + iconDataName);
			if(iconDataFile.exists()) {
				icon = new ImageIcon("./" + iconDataName);
				img = icon.getImage();
			}
			// try to find it inside a jar archive....
			else {
				is = ALDChooseOpNameFrame.class.getResourceAsStream(iconDataName);
				if (is == null) {
					System.err.println("Warning - cannot find icons...");
					img = new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
				}
				else {
					img = ImageIO.read(is);
				}
				bi= new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
				g = bi.createGraphics();
				g.drawImage(img, 0, 0, 20, 20, null);
			}
		} catch (IOException ex) {
			System.err.println("ALDChooseOpNameFrame - problems loading icons...!");
			img = new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
			bi= new BufferedImage(20,20,BufferedImage.TYPE_INT_ARGB);
			g = bi.createGraphics();
			g.drawImage(img, 0, 0, 20, 20, null);
		}
		this.aboutIcon = new ImageIcon(img);
	}
	
	@Override
	protected MTBOperatorControlFrame getOpControlFrame(String name) {
  	MTBOperatorControlFrame frame = null;
		try {
			frame = new MTBOperatorControlFrame(name);
		} catch (ALDOperatorException e) {
			Object[] okOption = { "OK" };
			JOptionPane.showOptionDialog(
					null,
					"Could not instante chosen operator \"" + name + "\"\n" +
				  "Something seems to be wrong with its class implementation...",
				  "Warning", JOptionPane.DEFAULT_OPTION,
				  JOptionPane.WARNING_MESSAGE, null, okOption, okOption[0]);
		}
  	return frame;
	}
	
	@Override
	protected JMenu generateHelpMenu() {
		JMenu helpM = new JMenu("Help");
		JMenuItem itemHelp = new JMenuItem("Online Help");
		itemHelp.addActionListener(
				OnlineHelpDisplayer.getHelpActionListener(itemHelp,"welcome"));
		JMenuItem itemAbout = new JMenuItem("About MiToBo");
		itemAbout.setActionCommand("showAbout");
		itemAbout.addActionListener(this);
		helpM.add(itemHelp);
		helpM.add(itemAbout);
		return helpM;
	}
	
	@Override
  protected void showAboutBox() {
		Object[] options = { "OK" };
		String msg = "<html>MiToBo - A Microscope Image Analysis Toolbox, " 
	    + "Release 1.0<p><p>" + "\u00a9 2010 - 2012   "
	    + "Martin Luther University Halle-Wittenberg<p>"
	    + "Institute of Computer Science, Faculty of Natural Sciences III<p><p>"
	    + "Email: mitobo@informatik.uni-halle.de<p>"
	    + "Internet: <i>www.informatik.uni-halle.de/mitobo</i><p>"
	    + "License: GPL 3.0, <i>http://www.gnu.org/licenses/gpl.html</i></html>";

		JOptionPane.showOptionDialog(null, new JLabel(msg),
		    "Information about MiToBo", JOptionPane.DEFAULT_OPTION,
		    JOptionPane.INFORMATION_MESSAGE, this.aboutIcon, options, options[0]);
	}
}
