/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 4623 $
 * $Date: 2011-11-29 09:04:25 +0100 (Di, 29 Nov 2011) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.snakes.energies;



import de.unihalle.informatik.Alida.annotations.ALDDataIOProvider;
import de.unihalle.informatik.Alida.dataio.provider.helpers.ALDParametrizedClassConfigWindow;
import de.unihalle.informatik.Alida.dataio.provider.swing.ALDDataIOSwing;
import de.unihalle.informatik.Alida.dataio.provider.swing.components.ALDSwingComponent;
import de.unihalle.informatik.Alida.dataio.provider.swing.events.ALDSwingValueChangeEvent;
import de.unihalle.informatik.Alida.dataio.provider.swing.events.ALDSwingValueChangeListener;
import de.unihalle.informatik.Alida.dataio.provider.swing.events.ALDSwingValueChangeReporter;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException;
import de.unihalle.informatik.Alida.exceptions.ALDDataIOProviderException.ALDDataIOProviderExceptionType;
import de.unihalle.informatik.Alida.helpers.ALDClassInfo;

import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;

import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Field;
import java.util.*;

/**
 * Class for loading/saving sets of PDE snake energies in GUI contexts.
 * 
 * @author moeller
 */
@ALDDataIOProvider(priority=10)

public class MTBSet_SnakeEnergyDerivableDataIOSwing implements ALDDataIOSwing {

	/**
	 * Interface method to announce class for which IO is provided for.
	 * 
	 * @return  Collection of classes provided.
	 */
	@Override
	public Collection<Class<?>> providedClasses() {
		LinkedList<Class<?>> classes = new LinkedList<Class<?>>();
		classes.add(MTBSet_SnakeEnergyDerivable.class);
		return classes;
	}
	
	/** 
	 * GUI element for reading sets of snake energies.
	 */
	/* (non-Javadoc)
	 * @see de.unihalle.informatik.Alida.helpers.ALDDataIOSwing#createGUIElement(java.lang.Class, java.lang.Object)
	 */
	@Override
	public ALDSwingComponent createGUIElement(
			Field field, Class<?> cl, Object obj) {
		return new MTBSet_SnkEnergyPDEConfigButton(field, cl, obj);
	}

	@Override
  public void setValue(
  		Field field, Class<?> cl, ALDSwingComponent guiElement, Object value) 
  	throws ALDDataIOProviderException {
		if (!(guiElement instanceof MTBSet_SnkEnergyPDEConfigButton))
			throw new ALDDataIOProviderException(
      		ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
      		"MTBSet_SnkEnergy-IO: setValue() got invalid GUI element!");
		if (!(value instanceof MTBSet_SnakeEnergyDerivable))
			throw new ALDDataIOProviderException(
      		ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
      		"MTBSet_SnkEnergy-IO: setValue() got wrong object type!");
		try {
			((MTBSet_SnkEnergyPDEConfigButton)guiElement).setValue(field, cl, value);
		} catch (ALDDataIOException exp) {
			throw new ALDDataIOProviderException(
      		ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
      		"MTBSet_SnkEnergy-IO: setValue() an error occurred...\n" +
      		exp.getCommentString());
		}
  }

	@Override
	public Object readData(Field field, Class<?> cl, ALDSwingComponent guiElement) 
		throws ALDDataIOProviderException {
		if (!(guiElement instanceof MTBSet_SnkEnergyPDEConfigButton))
			return null;
		try {
			Object obj =
				((MTBSet_SnkEnergyPDEConfigButton)guiElement).readData(field, cl);
			return obj;
    } catch (ALDDataIOException e) {
    	// if exception came from provider, just throw it to parent
    	if (e instanceof ALDDataIOProviderException)
    		throw (ALDDataIOProviderException)e;
    	// otherwise embed it into our own exception
    	// (... necessary due to method signature)
			throw new ALDDataIOProviderException(
      		ALDDataIOProviderExceptionType.UNSPECIFIED_ERROR, 
      		"MTBSet_SnkEnergy-IO: Data IO Manager returned an error: \n" +
      		e.getCommentString());
    }
	}

	@Override
	public JComponent writeData(Object obj) {
		// it is very, very unlikely that set of energies will be defined as
		// output parameters of operators, thus, the functionality to write this
		// data type is considered unnecessary for the moment...
		Object[] options = { "OK" };
		JOptionPane.showOptionDialog(null, 
				"MTBSet_SnkEnergyPDEDataIOSwing: \n" + 
				"You are trying to display an instance of this class \n" +
				"to the user. This usecase has not been considered so far.\n" +
				"Please contact {alida,mitobo}@informatik.uni-halle.de", "Warning",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);
		return null;
	}

	/**
	 * GUI element for configuring collections.
	 * <p>
	 * This button has a collection configuration window attached to it 
	 * where specific data is stored and accessable.
	 * 
	 * @author moeller
	 */
	private class MTBSet_SnkEnergyPDEConfigButton extends ALDSwingComponent 
		implements ALDSwingValueChangeListener {

		/**
		 * Button to display configuration window.
		 */
		private JButton confButton;
		
		/**
		 * Collection configuration window.
		 */
		private MTBSet_SnkEnergyPDEConfigWindow confWin;

		/**
		 * Constructor.
		 * 
		 * @param field	Field of collection.
		 * @param cl	Class of collection.
		 * @param obj	Default object.
		 */
		public MTBSet_SnkEnergyPDEConfigButton(
																			Field field, Class<?> cl, Object obj) {
			this.confWin = new MTBSet_SnkEnergyPDEConfigWindow(field, cl, obj);
			this.confWin.addValueChangeEventListener(this);
			this.confButton = new JButton("Configure Energies...");
			this.confButton.setActionCommand("configButtonPressed");
			this.confButton.addActionListener(this.confWin);
		}
		
		@Override
    public JButton getJComponent() {
			return this.confButton;
		}
		
		/**
		 * Updates GUI component with given value.
		 * 
		 * @param field	Field of collection.
		 * @param cl	Class of collection.
		 * @param obj	Default object.
		 * @return Current data.
		 * @throws ALDDataIOException 
		 */
		public void setValue(Field field, Class<?> cl, Object obj) 
			throws ALDDataIOException {
			this.confWin.setValue(field, cl,obj);
		}
		
		/**
		 * Gets the data from the configuration window.
		 * 
		 * @param field	Field of collection.
		 * @param cl	Class of collection.
		 * @param obj	Default object.
		 * @return Current data.
		 * @throws ALDDataIOException 
		 */
		public Object readData(Field field, Class<?> cl) throws ALDDataIOException {
			return this.confWin.readData(field, cl);
		}

		@Override
    public void handleValueChangeEvent(ALDSwingValueChangeEvent event) {
			System.out.println("Something changed in Snake energies...");
			this.fireALDSwingValueChangeEvent(event);
		}
	}	
	
	/**
	 * Collection configuration window.
	 * 
	 * @author moeller
	 */
	private class MTBSet_SnkEnergyPDEConfigWindow 
		extends ALDSwingValueChangeReporter	implements ActionListener, 
														TableModelListener, ALDSwingValueChangeListener {

		/**
		 * Main frame.
		 */
		private JFrame window;

		/**
		 * Fixed width of window.
		 */
		private final int frameWidth = 400;
		
		/**
		 * Fixed height of window.
		 */
		private final int frameHeight = 450;

		/**
		 * Main panel of main frame.
		 */
		private JPanel mainPanel = null;

		/**
		 * List of available energies.
		 */
		@SuppressWarnings("rawtypes")
		private Collection<Class> availableClasses = null;
		
		/**
		 * Mapping of short names to class objects.
		 */
		@SuppressWarnings("rawtypes")
		private HashMap<String, Class> classNameMapping = null;
		
		/**
		 * List of currently selected energies.
		 */
		private LinkedList<String> selectedEnergies =	new LinkedList<String>();
		
		/**
		 * Mapping of short names to configuration windows.
		 */
//		private HashMap<String, MTBSnkEnergyConfigWindow> energyConfWins = null;
		private HashMap<String, ALDParametrizedClassConfigWindow> 
			energyConfWins = null;

		/**
		 * Default object.
		 */
		private Object myobject = null;
		
		/**
		 * Add button for energies.
		 */
		private JButton addEnergyButton;
		/**
		 * Remove button for energies.
		 */
		private JButton removeEnergyButton;
		/**
		 * Config button for energies.
		 */
		private JButton confEnergyButton;
		/**
		 * Close button.
		 */
		private JButton closeButton;
		/**
		 * Combobox for selecting energies.
		 */
		private JList energySelection;
		/**
		 * List of selected energies.
		 */
		JTable energyTab = null;
		/**
		 * Model for the energy table.
		 */
		MTBSet_SnkEnergyPDEGUITableModel energyTabModel;
		/**
		 * Individual energy weights.
		 */
		Vector<Double> energyWeights = new Vector<Double>();

		/**
		 * Default constructor.
		 * 
		 * @param f	Field.
		 * @param c	Class.
		 * @param o	Preset object.
		 */
		public MTBSet_SnkEnergyPDEConfigWindow(Field f, Class<?> c, Object o) {
			this.myobject = o;
			this.window = new JFrame();
			this.window.setTitle("MTB Snake Derivable Energies");
			this.window.setSize(this.frameWidth,this.frameHeight);
			this.buildMainPanel();
			this.window.add(this.mainPanel);
		}
		
		/**
		 * Extracts current collection data.
		 * 
		 * @param field	Field of collection elements.
		 * @param cl	Class of collection elements.
		 * @return	Current collection.
		 * @throws ALDDataIOException 
		 */
		@SuppressWarnings("unused")
    public void setValue(Field field, Class<?> cl, Object value) 
			throws ALDDataIOException {
			MTBSet_SnakeEnergyDerivable enerSet = (MTBSet_SnakeEnergyDerivable)value;
			// new table model
			this.selectedEnergies.clear();
			this.energyTabModel= new MTBSet_SnkEnergyPDEGUITableModel(0, 2);
			this.energyTabModel.addTableModelListener(this);
			this.energyTabModel.setColumnIdentifiers(new Object[]{"Energy","Weight"});
			for (int i=0; i<enerSet.getEnergyList().size();++i) {
				MTBSnakeEnergyDerivable e = enerSet.getEnergy(i);
				// update energy object itself and its configuration window
				this.energyConfWins.get(e.getClass().getSimpleName()).setValue(e);
				// update list of selected energies
				Double weight = enerSet.getWeight(i);
				String energy = e.getClass().getSimpleName();
				if (this.selectedEnergies.contains(energy))
					return;
				// add new energy to selection
				this.selectedEnergies.add(energy);
				Object [] newRow = new Object[]{energy,weight.toString()};
				this.energyWeights.add(weight);
				this.energyTabModel.addRow(newRow);
			}	
			this.energyTab.setModel(this.energyTabModel);
			this.energyTabModel.fireTableDataChanged();
		}
		
		/**
		 * Extracts current collection data.
		 * 
		 * @param field	Field of collection elements.
		 * @param cl	Class of collection elements.
		 * @return	Current collection.
		 * @throws ALDDataIOException 
		 */
		public MTBSet_SnakeEnergyDerivable readData(Field field, Class<?> cl) 
			throws ALDDataIOException {
			// get number of entries in table
			int rows = this.energyTab.getRowCount();
			if (rows == 0)
				return null;
			
			MTBSet_SnakeEnergyDerivable snakeSet = new MTBSet_SnakeEnergyDerivable();
	    Vector<Double> vecWeights = (Vector<Double>)this.energyWeights.clone();
			for (int r = 0; r < rows; ++r) {
				String ename = (String)this.energyTabModel.getValueAt(r, 0);
				Class<?> c = this.classNameMapping.get(ename);
        MTBSnakeEnergyDerivable energy = 
        	(MTBSnakeEnergyDerivable)this.energyConfWins.get(ename).readData(field,c);
        snakeSet.addEnergy(energy, vecWeights.get(r));
			}
			return snakeSet;
		}
		
		/**
		 * Displays configuration window.
		 * <p>
		 * If a default collection is given, the collection is displayed.
		 */
		private void openWindow() {
			this.window.setVisible(true);
		}

		/**
		 * Build the main panel for configuring the list of energies.
		 */
		@SuppressWarnings("rawtypes")
		private void buildMainPanel() {

			this.mainPanel = new JPanel();
			BoxLayout pgr = new BoxLayout(this.mainPanel, BoxLayout.Y_AXIS);
			this.mainPanel.setLayout(pgr);

			// temporary local variables
			JLabel tmpLab = null;
			JPanel tmpPanel = null;
			FlowLayout fl = null;

			fl = new FlowLayout();
			fl.setAlignment(FlowLayout.LEFT);
			tmpPanel = new JPanel();
			tmpPanel.setLayout(fl);
			tmpLab = new JLabel("Available energies: ");
			tmpPanel.add(tmpLab);
			this.mainPanel.add(tmpPanel);

			// list of available energies
			fl = new FlowLayout();
			fl.setAlignment(FlowLayout.LEFT);
			tmpPanel = new JPanel();
			tmpPanel.setLayout(fl);
			this.availableClasses= 
				ALDClassInfo.lookupExtendingClasses(MTBSnakeEnergyDerivable.class);
			
			this.classNameMapping = new HashMap<String, Class>();
			this.energyConfWins = 
				new HashMap<String, ALDParametrizedClassConfigWindow>();
			Vector<String> energyList = new Vector<String>();
			for (Class c : this.availableClasses) {
				String cname = c.getSimpleName();
				this.classNameMapping.put(cname, c);
				ALDParametrizedClassConfigWindow tmpWin =
						new ALDParametrizedClassConfigWindow(c);
				tmpWin.addValueChangeEventListener(this);
				this.energyConfWins.put(cname, tmpWin); 
				energyList.add(cname);
			}
			// sort list of available energies lexicographically
			Collections.sort(energyList);
			
			this.energySelection = new JList(energyList);
			this.energySelection.ensureIndexIsVisible(1);
			this.energySelection.setSelectionMode(
																				ListSelectionModel.SINGLE_SELECTION);
			this.energySelection.setEnabled(true);
			JScrollPane scrollPane = new JScrollPane();
			scrollPane.setViewportView(this.energySelection);
			scrollPane.setAutoscrolls(true);
			this.mainPanel.add(scrollPane);
			
			tmpPanel = new JPanel();
			this.addEnergyButton = new JButton("   Add selected energy...   ");
			this.addEnergyButton.setActionCommand("addEnergy");
			this.addEnergyButton.addActionListener(this);
			tmpPanel.add(this.addEnergyButton);
			this.mainPanel.add(tmpPanel);

			fl = new FlowLayout();
			fl.setAlignment(FlowLayout.LEFT);
			tmpPanel = new JPanel();
			tmpPanel.setLayout(fl);
			tmpLab = new JLabel("Selected energies: ");
			tmpPanel.add(tmpLab);
			this.mainPanel.add(tmpPanel);

			// selected energies table
			fl = new FlowLayout();
			fl.setAlignment(FlowLayout.LEFT);
			tmpPanel = new JPanel();
			tmpPanel.setLayout(fl);
			this.energyTabModel= new MTBSet_SnkEnergyPDEGUITableModel(0, 2);
			this.energyTabModel.addTableModelListener(this);
			this.energyTabModel.setColumnIdentifiers(new Object[]{"Energy","Weight"});
			this.energyTab = new JTable(this.energyTabModel);
			scrollPane= new JScrollPane();
			scrollPane.setViewportView(this.energyTab);
			scrollPane.setAutoscrolls(true);
			this.mainPanel.add(scrollPane);

			// add remove and configure buttons
			GridLayout gl = new GridLayout(1,2);
			tmpPanel = new JPanel();
			tmpPanel.setLayout(gl);
			this.confEnergyButton = new JButton(" Configure  ");
			this.confEnergyButton.setActionCommand("configEnergy");
			this.confEnergyButton.addActionListener(this);
			this.removeEnergyButton = new JButton("     Remove       ");
			this.removeEnergyButton.setActionCommand("removeEnergy");
			this.removeEnergyButton.addActionListener(this);
			tmpPanel.add(this.removeEnergyButton);
			tmpPanel.add(this.confEnergyButton);
			this.mainPanel.add(tmpPanel);

			// close button
			gl = new GridLayout(1,2);
			tmpPanel = new JPanel();
			tmpPanel.setLayout(gl);
			this.closeButton = new JButton(" Close  ");
			this.closeButton.setActionCommand("closeWin");
			this.closeButton.addActionListener(this);
			tmpPanel.add(new JLabel(""));
			tmpPanel.add(this.closeButton);
			this.mainPanel.add(tmpPanel);
		}

		@Override
    public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand(); 
			if (cmd.equals("configButtonPressed")) {
				this.openWindow();
			}
			// handle configuration actions
			if (e.getActionCommand() == "addEnergy") {
				// get ID from GUI
				String energy = (String)this.energySelection.getSelectedValue();
				if (this.selectedEnergies.contains(energy))
					return;
				// add new energy to selection
				this.selectedEnergies.add(energy);
				Object [] newRow = new Object[]{energy,"1.0"};
				this.energyWeights.add(new Double(1.0));
				this.energyTabModel.addRow(newRow);
				this.energyTabModel.fireTableDataChanged();
			}
			if (e.getActionCommand() == "removeEnergy") {
				// get selected row from table
				if (this.energyTab.getSelectedRow() != -1) {
					int entry = this.energyTab.getSelectedRow();
					this.selectedEnergies.remove(entry);
					this.energyWeights.remove(entry);
					this.energyTabModel.removeRow(entry);
					this.energyTabModel.fireTableDataChanged();
				}
			}
			if (e.getActionCommand() == "configEnergy") {
				// get selected row from table
				if (this.energyTab.getSelectedRow() != -1) {
					int entry = this.energyTab.getSelectedRow();
					String energy = (String)this.energyTabModel.getValueAt(entry, 0);
					// open the corresponding window
					this.energyConfWins.get(energy).setVisible(true);
				}
			}
			else if (cmd.equals("closeWin")) {
				this.window.setVisible(false);
			}
	  }		
		
		@Override
    public void tableChanged(TableModelEvent e) {
	    this.fireALDSwingValueChangeEvent(new ALDSwingValueChangeEvent(this));
    }

		@Override
    public void handleValueChangeEvent(ALDSwingValueChangeEvent event) {
			this.fireALDSwingValueChangeEvent(event);
		}

		/**
		 * Internal PDE snake energy GUI table model.
		 * 
		 * @author moeller
		 */
		private class MTBSet_SnkEnergyPDEGUITableModel extends DefaultTableModel
		implements TableModelListener {

			/**
			 * Default serial number.
			 */
			private static final long serialVersionUID = 1L;

			/**
			 * Default constructor.
			 * 
			 * @param row	Initial row count.
			 * @param col	Initial col count.
			 */
			public MTBSet_SnkEnergyPDEGUITableModel(int row, int col) {
				super(row,col);
				this.addTableModelListener(this);
			}

			@Override
			public boolean isCellEditable(int row, int column) {
				if (column == 0)
					return false;
				return super.isCellEditable(row, column);
			}

			@Override
			public void tableChanged(TableModelEvent e) {
				int row = e.getFirstRow();
				if (e.getType() == TableModelEvent.UPDATE && row != -1) {
					if (MTBSet_SnkEnergyPDEConfigWindow.
								this.energyTabModel.getRowCount() > 0) {
						Double d = Double.valueOf(
								(String)MTBSet_SnkEnergyPDEConfigWindow.
								this.energyTabModel.getValueAt(row, 1));
						MTBSet_SnkEnergyPDEConfigWindow.
								this.energyWeights.setElementAt(d, row);
					}
				}
				if (MTBSet_SnkEnergyPDEConfigWindow.this.energyTab != null)	
					MTBSet_SnkEnergyPDEConfigWindow.this.energyTab.tableChanged(e);
			}
		}
	}
}
