/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5762 $
 * $Date: 2012-07-06 16:07:32 +0200 (Fr, 06. Jul 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.snakes.energies;

import Jama.Matrix;



import de.unihalle.informatik.Alida.annotations.ALDClassParameter;
import de.unihalle.informatik.Alida.annotations.ALDDerivedClass;
import de.unihalle.informatik.Alida.annotations.ALDParametrizedClass;
import de.unihalle.informatik.Alida.annotations.Parameter.ExpertMode;
import de.unihalle.informatik.MiToBo.core.datatypes.defines.MTBConstants;
import de.unihalle.informatik.MiToBo.segmentation.basics.MTBSegmentationInterface;
import de.unihalle.informatik.MiToBo.segmentation.snakes.datatypes.MTBSnake;
import de.unihalle.informatik.MiToBo.segmentation.snakes.energies.paramAdapt.MTBSnakeEnergyCD_KassInt_ParamAdapt;
import de.unihalle.informatik.MiToBo.segmentation.snakes.energies.paramAdapt.MTBSnakeEnergyCD_KassInt_ParamAdaptNone;
import de.unihalle.informatik.MiToBo.segmentation.snakes.optimize.*;

/**
 * Internal snake energy of the pioneering paper of Kass et al.
 * <p>
 * This energy includes a length term based on first order derivatives of the
 * snake and a curvature term based on second order derivatives of the snake. In
 * contrast to the original work, however, it allows for more flexible weighting
 * parameters alpha and beta, i.e. the weighting factors may be set for each
 * snake point individually.
 * <p>
 * In addition, the weighting parameters can dynamically be adjusted by
 * providing the energy with an update object, see
 * {@link MTBSnakeEnergyCD_KassInt_ParamAdapt}.
 * 
 * @author moeller
 * @deprecated
 */
//@ALDDerivedClass
@ALDParametrizedClass

@Deprecated
public class MTBSnakeEnergyCD_KassInt 
	implements MTBSnakeEnergyDerivable, MTBSnakeEnergyComputable {

	/**
	 * Weighting factor for the length term.
	 */
	@ALDClassParameter(label="Initial Alpha", mode = ExpertMode.STANDARD,
			dataIOOrder = -10)
	private double alphaInitial = 1.0;
	
	/**
	 * Weighting factor for the curvature term.
	 */
	@ALDClassParameter(label="Initial Beta", mode = ExpertMode.STANDARD,
			dataIOOrder = -5)
	private double betaInitial = 1.0;

	/**
	 * Parameter update strategy.
	 */
	@ALDClassParameter(label="Parameter Updater", mode = ExpertMode.ADVANCED,
			dataIOOrder = 0)
	private MTBSnakeEnergyCD_KassInt_ParamAdapt pUpdater =
		new MTBSnakeEnergyCD_KassInt_ParamAdaptNone();

	/**
	 * Maximal possible value of alpha.
	 */
	private transient Double maxAlpha;
	
	/**
	 * Minimal possible value of alpha.
	 */
	private transient Double minAlpha;
	
	/**
	 * Maximal possible value of beta.
	 */
	private transient Double maxBeta;
	
	/**
	 * Minimal possible value of alpha.
	 */
	private transient Double minBeta;
	
//	private double maxEnergy;
//	
//	private double minEnergy;

	private transient double maxEnergyAlpha;
	
	private transient double minEnergyAlpha;
	
	private transient double maxEnergyBeta;
	
	private transient double minEnergyBeta;

//	private double energyRange;
	
	private transient double energyRangeAlpha;

	private transient double energyRangeBeta;

	private transient double targetEnergyRange = 2;
	
	private transient double targetEnergyMin = -1;

	private transient boolean normalizationImpossible = false;
	
	/**
	 * Vector of point-specific alpha values.
	 */
	private transient double[] alphas;

	/**
	 * Vector of point-specific beta values.
	 */
	private transient double[] betas;

	/**
	 * Default constructor.
	 */
	public MTBSnakeEnergyCD_KassInt() {
		// nothing to do here
	}
	
	/**
	 * Constructor with dynamic parameter updater.
	 * 
	 * @param _alpha
	 *          Weighting factor for length term.
	 * @param _beta
	 *          Weighting factor for curvature term.
	 * @param _pUpd
	 *          Parameter update object.
	 */
	public MTBSnakeEnergyCD_KassInt(double _alpha, double _beta,
			MTBSnakeEnergyCD_KassInt_ParamAdapt _pUpd) {
		this.alphaInitial = _alpha;
		this.betaInitial = _beta;
		this.pUpdater = _pUpd;
	}

		/**
		 * Scaling factor to rescale image coordinates in range [0,1] or several
		 * ranges to the original coordinates range like [1000, 1000] in a image of
		 * size 1000 x 1000. Default scaling factor is 1.
		 */
		protected transient double scaleFactor = 1.0;

		/**
		 * Set the scaling factor.
		 * 
		 * @param s
		 *          new scaling factor.
		 */
		@Override
		public void setScaleFactor(double s) {
				this.scaleFactor = s;
		}

		/**
		 * Get scaling factor.
		 * 
		 * @return Scaling factor.
		 */
		@Override
		public double getScaleFactor() {
				return this.scaleFactor;
		}

		@Override
		public boolean initEnergy(SnakeOptimizerSingleVarCalc o) {
//				MTBSnake s = o.getCurrentSnake();
//				if (s == null)
//						return true;
//				int pointNum = s.getPointNum();
//				if (pointNum > 0) {
//						this.alphas = new double[pointNum];
//						this.betas = new double[pointNum];
			this.alphas = new double[1];
			this.betas = new double[1];
			this.alphas[0] = this.alphaInitial;
			this.betas[0] = this.betaInitial;
//				}
				this.maxAlpha = this.pUpdater.getMaxAlpha();
				this.minAlpha = this.pUpdater.getMinAlpha();
				this.maxBeta = this.pUpdater.getMaxBeta();
				this.minBeta = this.pUpdater.getMinBeta();
				double minE=0 , maxE= 0;
//				if (!this.minAlpha.isNaN() && !this.minBeta.isNaN())
//					minE = -this.minAlpha.doubleValue() - 4*this.minBeta.doubleValue();
//				else 
//					this.normalizationImpossible= true;
//				if (!this.maxAlpha.isNaN() && !this.maxBeta.isNaN())
//					maxE = 2*this.maxAlpha.doubleValue() + 6*this.maxBeta.doubleValue();
//				else 
//					this.normalizationImpossible= true;
//				if (Math.abs(minE) > Math.abs(maxE)) {
//					this.minEnergy = -Math.abs(minE);
//					this.maxEnergy = Math.abs(minE);
//				}
//				else {
//					this.minEnergy = -Math.abs(maxE);
//					this.maxEnergy = Math.abs(maxE);					
//				}
//				this.energyRange = this.maxEnergy - this.minEnergy;
				if (!this.minAlpha.isNaN() && !this.maxAlpha.isNaN()) {
					minE = - 2 * this.minAlpha.doubleValue();
					maxE =   2 * this.maxAlpha.doubleValue();
					if (Math.abs(minE) > Math.abs(maxE)) {
						this.minEnergyAlpha = -Math.abs(minE);
						this.maxEnergyAlpha = Math.abs(minE);
					}
					else {
						this.minEnergyAlpha = -Math.abs(maxE);
						this.maxEnergyAlpha = Math.abs(maxE);					
					}
				}
				this.energyRangeAlpha = this.maxEnergyAlpha - this.minEnergyAlpha;
				if (!this.minBeta.isNaN() && !this.maxBeta.isNaN()) {
					minE = - 8 * this.minBeta.doubleValue();
					maxE =   8 * this.maxBeta.doubleValue();
					if (Math.abs(minE) > Math.abs(maxE)) {
						this.minEnergyBeta = -Math.abs(minE);
						this.maxEnergyBeta =  Math.abs(minE);
					}
					else {
						this.minEnergyBeta = -Math.abs(maxE);
						this.maxEnergyBeta =  Math.abs(maxE);					
					}
				}
				this.energyRangeBeta = this.maxEnergyBeta - this.minEnergyBeta;
				return true;
		}

		@Override
		public boolean initEnergy(SnakeOptimizerSingleGreedy o) {
//				MTBSnake s = o.getCurrentSnake();
//				if (s == null)
//						return true;
//				int pointNum = s.getPointNum();
//				if (pointNum > 0) {
//						this.alphas = new double[pointNum];
//						this.betas = new double[pointNum];
			this.alphas = new double[1];
			this.betas = new double[1];
			this.alphas[0] = this.alphaInitial;
			this.betas[0] = this.betaInitial;
//				}
				this.maxAlpha = this.pUpdater.getMaxAlpha();
				this.minAlpha = this.pUpdater.getMinAlpha();
				this.maxBeta = this.pUpdater.getMaxBeta();
				this.minBeta = this.pUpdater.getMinBeta();
				double minE=0 , maxE= 0;
//				if (!this.minAlpha.isNaN() && !this.minBeta.isNaN())
//					minE = -this.minAlpha.doubleValue() - 4*this.minBeta.doubleValue();
//				else 
//					this.normalizationImpossible= true;
//				if (!this.maxAlpha.isNaN() && !this.maxBeta.isNaN())
//					maxE = 2*this.maxAlpha.doubleValue() + 6*this.maxBeta.doubleValue();
//				else 
//					this.normalizationImpossible= true;
//				if (Math.abs(minE) > Math.abs(maxE)) {
//					this.minEnergy = -Math.abs(minE);
//					this.maxEnergy = Math.abs(minE);
//				}
//				else {
//					this.minEnergy = -Math.abs(maxE);
//					this.maxEnergy = Math.abs(maxE);					
//				}
//				this.energyRange = this.maxEnergy - this.minEnergy;
				if (!this.minAlpha.isNaN() && !this.maxAlpha.isNaN()) {
					minE = - 2 * this.minAlpha.doubleValue();
					maxE =   2 * this.maxAlpha.doubleValue();
					if (Math.abs(minE) > Math.abs(maxE)) {
						this.minEnergyAlpha = -Math.abs(minE);
						this.maxEnergyAlpha = Math.abs(minE);
					}
					else {
						this.minEnergyAlpha = -Math.abs(maxE);
						this.maxEnergyAlpha = Math.abs(maxE);					
					}
				}
				this.energyRangeAlpha = this.maxEnergyAlpha - this.minEnergyAlpha;
				if (!this.minBeta.isNaN() && !this.maxBeta.isNaN()) {
					minE = - 8 * this.minBeta.doubleValue();
					maxE =   8 * this.maxBeta.doubleValue();
					if (Math.abs(minE) > Math.abs(maxE)) {
						this.minEnergyBeta = -Math.abs(minE);
						this.maxEnergyBeta =  Math.abs(minE);
					}
					else {
						this.minEnergyBeta = -Math.abs(maxE);
						this.maxEnergyBeta =  Math.abs(maxE);					
					}
				}
				this.energyRangeBeta = this.maxEnergyBeta - this.minEnergyBeta;
				return true;
		}

		/**
		 * Get the initial alpha value.
		 * 
		 * @return Intial alpha.
		 */
		public double getInitAlpha() {
				return this.alphaInitial;
		}

		/**
		 * Get the initial beta value.
		 * 
		 * @return Intial beta.
		 */
		public double getInitBeta() {
				return this.betaInitial;
		}

		/**
		 * Calculates length term of snake energy.
		 */
		@Override
		public double calcEnergy(SnakeOptimizerSingle opt) {

				MTBSnake snake = opt.getCurrentSnake();

				double lEnergy = 0.0;

				int pointNum = snake.getPointNum();

				// File output;
				// FileWriter fw;
				// BufferedWriter bw;

				for (int i = 0; i < pointNum; ++i) {
						double x_diff = snake.getPartialDiffX(i);
						double y_diff = snake.getPartialDiffY(i);
						lEnergy += this.alphas[i] * (x_diff * x_diff + y_diff * y_diff);
						// input.insert(input.length(), (x_diff * x_diff + y_diff * y_diff) +
						// "\t");
				}

				double cEnergy = 0.0;
				for (int i = 0; i < pointNum; ++i) {
						double x_diff = snake.getSndPartialDiffX(i);
						double y_diff = snake.getSndPartialDiffY(i);
						cEnergy += this.betas[i] * (x_diff * x_diff + y_diff * y_diff);
				}
				return lEnergy + cEnergy;

				// input.insert(input.length(), "\n");

				// try {
				// output = new
				// File("C:/Users/Danny/Documents/SnakeTest/SnakeTest_intE.txt");
				// fw = new FileWriter(output);
				// bw = new BufferedWriter(fw);
				// bw.write(input.toString());
				// bw.close();
				// System.out.println("   --> energy saved to: " +
				// output.getAbsolutePath());
				// } catch (ArrayIndexOutOfBoundsException aioobe) {
				// System.out.println("Error writing energy: " + aioobe);
				// } catch (IOException ioe) {
				// System.out.println("Error writing energy: " + ioe);
				// }
		}

		/**
		 * Calculates length term of snake energy at a certain position for the
		 * current snake.
		 */
		public double calcEnergy(SnakeOptimizerSingle opt, int pos) {
				MTBSnake s = opt.getCurrentSnake();
				double x_diff = s.getPartialDiffX(pos);
				double y_diff = s.getPartialDiffY(pos);
				double lener = (this.alphas[pos] * (x_diff * x_diff + y_diff * y_diff));
				x_diff = s.getSndPartialDiffX(pos);
				y_diff = s.getSndPartialDiffY(pos);
				double cener = (this.betas[pos] * (x_diff * x_diff + y_diff * y_diff));
				return lener + cener;
		}

		/**
		 * Updates matrix A given weights for the internal energy term.
		 * <p>
		 * The upper left block and the lower right block of matrix A just dependent
		 * on the weight parameters alpha and beta for the internal snake energy
		 * terms.
		 */
		@Override
		public Matrix getDerivative_MatrixPart(SnakeOptimizerSingleVarCalc opt) {

				// get necessary data from the calling optimizer
				MTBSnake snake = opt.getCurrentSnake();
				int snakePointNum = snake.getPointNum();

				// update parameter vectors
				this.alphas = this.pUpdater.alphaUpdate(opt, this.alphas);
				this.betas = this.pUpdater.betaUpdate(opt, this.betas);

				if (this.alphas == null || this.betas == null)
						return null;

				// init result matrix
				Matrix A = new Matrix(snakePointNum * 2, snakePointNum * 2);
				
				/* upper left block */

				// first line
				// double oldVal= A.get(0,0);
				// A.set(0, 0, oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(0,0,this.normMatrixEntry(2 * this.alphas[0] + 6 * this.betas[0]));
				A.set(0,0,this.normMatrixEntryAlpha(2 * this.alphas[0]) 
							  + this.normMatrixEntryBeta(6 * this.betas[0]));
				// oldVal= A.get(0,1);
				// A.set(0, 1, oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(0, 1,this.normMatrixEntry( -this.alphas[0] - 4 * this.betas[0]));
				A.set(0, 1,this.normMatrixEntryAlpha( -this.alphas[0]) 
						     - this.normMatrixEntryBeta(4 * this.betas[0]));
				// oldVal= A.get(0,2);
				// A.set(0, 2, oldVal + this.betaInitial);
//				A.set(0, 2,this.normMatrixEntry( this.betas[0]));
				A.set(0, 2,this.normMatrixEntryBeta( this.betas[0]));
				// oldVal= A.get(0,snakePointNum - 2);
				// A.set(0, snakePointNum - 2, oldVal + this.betaInitial);
//				A.set(0, snakePointNum - 2,this.normMatrixEntry( this.betas[0]));
				A.set(0, snakePointNum - 2,this.normMatrixEntryBeta( this.betas[0]));
				// oldVal= A.get(0,snakePointNum - 1);
				// A.set(0, snakePointNum - 1, oldVal - this.alphaInitial - 4 *
				// this.betaInitial);
//				A.set(0, snakePointNum - 1,
//						this.normMatrixEntry( -this.alphas[0] - 4 * this.betas[0]));
				A.set(0, snakePointNum - 1,
						  this.normMatrixEntryAlpha( -this.alphas[0])
						- this.normMatrixEntryBeta(4 * this.betas[0]));
				// second line
				// oldVal= A.get(1,0);
				// A.set(1, 0, oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(1, 0,
//						this.normMatrixEntry( -this.alphas[1] - 4 * this.betas[1]));
				A.set(1, 0,
							this.normMatrixEntryAlpha( -this.alphas[1]) 
						- this.normMatrixEntryBeta(4 * this.betas[1]));
				// oldVal= A.get(1,1);
				// A.set(1, 1, oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(1, 1,
//						this.normMatrixEntry( 2 * this.alphas[1] + 6 * this.betas[1]));
				A.set(1, 1,
						  this.normMatrixEntryAlpha( 2 * this.alphas[1])
						+ this.normMatrixEntryBeta(6 * this.betas[1]));
				// oldVal= A.get(1,2);
				// A.set(1, 2, oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(1, 2,
//						this.normMatrixEntry( -this.alphas[1] - 4 * this.betas[1]));
				A.set(1, 2,
						  this.normMatrixEntryAlpha( -this.alphas[1] )
						- this.normMatrixEntryBeta(4 * this.betas[1]));
				// oldVal= A.get(1,3);
				// A.set(1, 3, oldVal + this.betaInitial);
//				A.set(1, 3,this.normMatrixEntry( this.betas[1]));
				A.set(1, 3,this.normMatrixEntryBeta( this.betas[1]));
				// oldVal= A.get(1,snakePointNum - 1);
				// A.set(1, snakePointNum - 1, oldVal + this.betaInitial);
//				A.set(1, snakePointNum - 1, this.normMatrixEntry(this.betas[1]));
				A.set(1, snakePointNum - 1, this.normMatrixEntryBeta(this.betas[1]));
				// inner lines
				for (int l = 2; l < snakePointNum - 2; ++l) {
						// oldVal= A.get(l,l-2);
						// A.set(l, l - 2, oldVal + this.betaInitial);
//						A.set(l, l - 2,this.normMatrixEntry( this.betas[l]));
						A.set(l, l - 2,this.normMatrixEntryBeta( this.betas[l]));
						// oldVal= A.get(l,l-1);
						// A.set(l, l - 1, oldVal - this.alphaInitial - 4 * this.betaInitial);
//						A.set(l, l - 1,
//								this.normMatrixEntry( -this.alphas[l] - 4 * this.betas[l]));
						A.set(l, l - 1,
								  this.normMatrixEntryAlpha( -this.alphas[l] )
								- this.normMatrixEntryBeta(4 * this.betas[l]));
						// oldVal= A.get(l,l);
						// A.set(l, l, oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//						A.set(l, l,
//								this.normMatrixEntry( 2 * this.alphas[l] + 6 * this.betas[l]));
						A.set(l, l,
								  this.normMatrixEntryAlpha( 2 * this.alphas[l] )
								+ this.normMatrixEntryBeta(6 * this.betas[l]));
						// oldVal= A.get(l,l+1);
						// A.set(l, l + 1, oldVal - this.alphaInitial - 4 * this.betaInitial);
//						A.set(l, l + 1, 
//								this.normMatrixEntry(-this.alphas[l] - 4 * this.betas[l]));
						A.set(l, l + 1, 
								  this.normMatrixEntryAlpha(-this.alphas[l])
								- this.normMatrixEntryBeta(4 * this.betas[l]));
						// oldVal= A.get(l,l+2);
						// A.set(l, l + 2, oldVal + this.betaInitial);
//						A.set(l, l + 2,this.normMatrixEntry( this.betas[l]));
						A.set(l, l + 2,this.normMatrixEntryBeta( this.betas[l]));
				}
				// pre-last line
				// oldVal= A.get(snakePointNum - 2,0);
				// A.set(snakePointNum - 2, 0, oldVal + this.betaInitial);
//				A.set(snakePointNum - 2, 0,
//						this.normMatrixEntry( this.betas[snakePointNum - 2]));
				A.set(snakePointNum - 2, 0,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 2]));
				// oldVal= A.get(snakePointNum - 2,snakePointNum - 4);
				// A.set(snakePointNum - 2, snakePointNum - 4, oldVal + this.betaInitial);
//				A.set(snakePointNum - 2, snakePointNum - 4,
//						this.normMatrixEntry( this.betas[snakePointNum - 2]));
				A.set(snakePointNum - 2, snakePointNum - 4,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 2]));
				// oldVal= A.get(snakePointNum - 2,snakePointNum - 3);
				// A.set(snakePointNum - 2, snakePointNum - 3,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum - 2, snakePointNum - 3,
//						this.normMatrixEntry( -this.alphas[snakePointNum - 2]
//				                          - 4 * this.betas[snakePointNum - 2]));
				A.set(snakePointNum - 2, snakePointNum - 3,
						this.normMatrixEntryAlpha( -this.alphas[snakePointNum - 2])
				  - this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 2]));
				// oldVal= A.get(snakePointNum - 2,snakePointNum - 2);
				// A.set(snakePointNum - 2, snakePointNum - 2,
				// oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(snakePointNum - 2, snakePointNum - 2, 
//						this.normMatrixEntry(2 * this.alphas[snakePointNum - 2] + 
//								                 6 * this.betas[snakePointNum - 2]));
				A.set(snakePointNum - 2, snakePointNum - 2, 
						this.normMatrixEntryAlpha(2 * this.alphas[snakePointNum - 2])  
					+	this.normMatrixEntryBeta(6 * this.betas[snakePointNum - 2]));
				// oldVal= A.get(snakePointNum - 2,snakePointNum - 1);
				// A.set(snakePointNum - 2, snakePointNum - 1,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum - 2, snakePointNum - 1,
//						this.normMatrixEntry( -this.alphas[snakePointNum - 2] 
//						                      - 4 * this.betas[snakePointNum - 2]));
				A.set(snakePointNum - 2, snakePointNum - 1,
						this.normMatrixEntryAlpha( -this.alphas[snakePointNum - 2]) 
					- this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 2]));
				// last line
				// oldVal= A.get(snakePointNum - 1,0);
				// A.set(snakePointNum - 1, 0, oldVal - this.alphaInitial - 4 *
				// this.betaInitial);
//				A.set(snakePointNum - 1, 0, 
//						this.normMatrixEntry(-this.alphas[snakePointNum - 1] - 4 
//								* this.betas[snakePointNum - 1]));
				A.set(snakePointNum - 1, 0, 
						this.normMatrixEntryAlpha(-this.alphas[snakePointNum - 1])
					- this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 1]));
				// oldVal= A.get(snakePointNum - 1,1);
				// A.set(snakePointNum - 1, 1, oldVal + this.betaInitial);
//				A.set(snakePointNum - 1, 1,
//						this.normMatrixEntry( this.betas[snakePointNum - 1]));
				A.set(snakePointNum - 1, 1,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 1]));
				// oldVal= A.get(snakePointNum - 1,snakePointNum - 3);
				// A.set(snakePointNum - 1, snakePointNum - 3, oldVal + this.betaInitial);
//				A.set(snakePointNum - 1, snakePointNum - 3,
//						this.normMatrixEntry( this.betas[snakePointNum - 1]));
				A.set(snakePointNum - 1, snakePointNum - 3,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 1]));
				// oldVal= A.get(snakePointNum - 1,snakePointNum - 2);
				// A.set(snakePointNum - 1, snakePointNum - 2,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum - 1, snakePointNum - 2,
//						this.normMatrixEntry( -this.alphas[snakePointNum - 1]
//				    - 4 * this.betas[snakePointNum - 1]));
				A.set(snakePointNum - 1, snakePointNum - 2,
						this.normMatrixEntryAlpha( -this.alphas[snakePointNum - 1])
				  - this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 1]));
				// oldVal= A.get(snakePointNum - 1,snakePointNum - 1);
				// A.set(snakePointNum - 1, snakePointNum - 1,
				// oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(snakePointNum - 1, snakePointNum - 1,
//						this.normMatrixEntry( 2 * this.alphas[snakePointNum - 1] 
//						                    + 6 * this.betas[snakePointNum - 1]));
				A.set(snakePointNum - 1, snakePointNum - 1,
						this.normMatrixEntryAlpha( 2 * this.alphas[snakePointNum - 1]) 
          + this.normMatrixEntryBeta(6 * this.betas[snakePointNum - 1]));

				/* lower right block */

				// first line
				// oldVal= A.get(snakePointNum,snakePointNum);
				// A.set(snakePointNum, snakePointNum,
				// oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(snakePointNum, snakePointNum,
//						this.normMatrixEntry( 2 * this.alphas[0] + 6 * this.betas[0]));
				A.set(snakePointNum, snakePointNum,
						this.normMatrixEntryAlpha( 2 * this.alphas[0]) 
					+ this.normMatrixEntryBeta(6 * this.betas[0]));
				// oldVal= A.get(snakePointNum,snakePointNum + 1);
				// A.set(snakePointNum, snakePointNum + 1,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum, snakePointNum + 1,
//						this.normMatrixEntry( -this.alphas[0] - 4 * this.betas[0]));
				A.set(snakePointNum, snakePointNum + 1,
						this.normMatrixEntryAlpha( -this.alphas[0])
					- this.normMatrixEntryBeta(4 * this.betas[0]));
				// oldVal= A.get(snakePointNum,snakePointNum + 2);
				// A.set(snakePointNum, snakePointNum + 2,
				// oldVal + this.betaInitial);
//				A.set(snakePointNum, snakePointNum + 2,
//						this.normMatrixEntry( this.betas[0]));
				A.set(snakePointNum, snakePointNum + 2,
						this.normMatrixEntryBeta( this.betas[0]));
				// oldVal= A.get(snakePointNum,2 * snakePointNum - 2);
				// A.set(snakePointNum, 2 * snakePointNum - 2,
				// oldVal + this.betaInitial);
//				A.set(snakePointNum, 2 * snakePointNum - 2, 
//						this.normMatrixEntry(this.betas[0]));
				A.set(snakePointNum, 2 * snakePointNum - 2, 
						this.normMatrixEntryBeta(this.betas[0]));
				// oldVal= A.get(snakePointNum,2 * snakePointNum - 1);
				// A.set(snakePointNum, 2 * snakePointNum - 1,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum, 2 * snakePointNum - 1,
//						this.normMatrixEntry( -this.alphas[0] - 4 * this.betas[0]));
				A.set(snakePointNum, 2 * snakePointNum - 1,
						this.normMatrixEntryAlpha( -this.alphas[0] )
					- this.normMatrixEntryBeta(4 * this.betas[0]));
				// second line
				// oldVal= A.get(snakePointNum + 1,snakePointNum);
				// A.set(snakePointNum + 1, snakePointNum,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum + 1, snakePointNum,
//						this.normMatrixEntry( -this.alphas[1] - 4* this.betas[1]));
				A.set(snakePointNum + 1, snakePointNum,
						this.normMatrixEntryAlpha( -this.alphas[1])
					- this.normMatrixEntryBeta(4* this.betas[1]));
				// oldVal= A.get(snakePointNum + 1,snakePointNum + 1);
				// A.set(snakePointNum + 1, snakePointNum + 1,
				// oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(snakePointNum + 1, snakePointNum + 1,
//						this.normMatrixEntry( 2 * this.alphas[1] + 6 * this.betas[1]));
				A.set(snakePointNum + 1, snakePointNum + 1,
						this.normMatrixEntryAlpha( 2 * this.alphas[1]) 
					+ this.normMatrixEntryBeta(6 * this.betas[1]));
				// oldVal= A.get(snakePointNum + 1,snakePointNum + 2);
				// A.set(snakePointNum + 1, snakePointNum + 2,
				// oldVal - this.alphaInitial - 4 * this.betaInitial);
//				A.set(snakePointNum + 1, snakePointNum + 2,
//						this.normMatrixEntry( -this.alphas[1] - 4 * this.betas[1]));
				A.set(snakePointNum + 1, snakePointNum + 2,
						this.normMatrixEntryAlpha( -this.alphas[1] )
					- this.normMatrixEntryBeta(4 * this.betas[1]));
				// oldVal= A.get(snakePointNum + 1,snakePointNum + 3);
				// A.set(snakePointNum + 1, snakePointNum + 3,
				// oldVal + this.betaInitial);
//				A.set(snakePointNum + 1, snakePointNum + 3,
//						this.normMatrixEntry( this.betas[1]));
				A.set(snakePointNum + 1, snakePointNum + 3,
						this.normMatrixEntryBeta( this.betas[1]));
				// oldVal= A.get(snakePointNum + 1,2 * snakePointNum - 1);
				// A.set(snakePointNum + 1, 2 * snakePointNum - 1,
				// oldVal + this.betaInitial);
//				A.set(snakePointNum + 1, 2 * snakePointNum - 1,
//						this.normMatrixEntry( this.betas[1]));
				A.set(snakePointNum + 1, 2 * snakePointNum - 1,
						this.normMatrixEntryBeta( this.betas[1]));
				// inner lines
				for (int l = snakePointNum + 2; l < 2 * snakePointNum - 2; ++l) {
						// oldVal= A.get(l,l-2);
						// A.set(l, l - 2, oldVal+this.betaInitial);
//						A.set(l, l - 2,
//								this.normMatrixEntry( this.betas[l - snakePointNum]));
						A.set(l, l - 2,
								this.normMatrixEntryBeta( this.betas[l - snakePointNum]));
						// oldVal= A.get(l,l-1);
						// A.set(l, l - 1, oldVal-this.alphaInitial - 4 * this.betaInitial);
//						A.set(l, l - 1,
//								this.normMatrixEntry( -this.alphas[l - snakePointNum] 
//								                      - 4 * this.betas[l - snakePointNum]));
						A.set(l, l - 1,
								this.normMatrixEntryAlpha( -this.alphas[l - snakePointNum]) 
							- this.normMatrixEntryBeta(4 * this.betas[l - snakePointNum]));
						// oldVal= A.get(l,l);
						// A.set(l, l, oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//						A.set(l, l, 
//								this.normMatrixEntry(2 * this.alphas[l - snakePointNum] + 
//										                 6 * this.betas[l - snakePointNum]));
						A.set(l, l, 
								this.normMatrixEntryAlpha(2 * this.alphas[l - snakePointNum]) 
							+ this.normMatrixEntryBeta(6 * this.betas[l - snakePointNum]));
						// oldVal= A.get(l,l+1);
						// A.set(l, l + 1, oldVal-this.alphaInitial - 4 * this.betaInitial);
//						A.set(l, l + 1,
//								this.normMatrixEntry( -this.alphas[l - snakePointNum] - 
//										                4* this.betas[l - snakePointNum]));
						A.set(l, l + 1,
								this.normMatrixEntryAlpha( -this.alphas[l - snakePointNum]) 
							- this.normMatrixEntryBeta(4* this.betas[l - snakePointNum]));
						// oldVal= A.get(l,l+2);
						// A.set(l, l + 2, oldVal+this.betaInitial);
//						A.set(l, l + 2,
//								this.normMatrixEntry( this.betas[l - snakePointNum]));
						A.set(l, l + 2,
								this.normMatrixEntryBeta( this.betas[l - snakePointNum]));
				}
				// pre-last line
				// oldVal= A.get(2 * snakePointNum - 2,snakePointNum);
				// A.set(2 * snakePointNum - 2, snakePointNum, oldVal+this.betaInitial);
//				A.set(2 * snakePointNum - 2, snakePointNum,
//						this.normMatrixEntry( this.betas[snakePointNum - 2]));
				A.set(2 * snakePointNum - 2, snakePointNum,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 2]));
				// oldVal= A.get(2 * snakePointNum - 2,2 * snakePointNum - 4);
				// A.set(2 * snakePointNum - 2, 2 * snakePointNum - 4,
				// oldVal+this.betaInitial);
//				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 4,
//						this.normMatrixEntry( this.betas[snakePointNum - 2]));
				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 4,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 2]));
				// oldVal= A.get(2 * snakePointNum - 2,2 * snakePointNum - 3);
				// A.set(2 * snakePointNum - 2, 2 * snakePointNum - 3,
				// oldVal-this.alphaInitial - 4 * this.betaInitial);
//				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 3,
//						this.normMatrixEntry( -this.alphas[snakePointNum - 2] 
//						                      - 4 * this.betas[snakePointNum - 2]));
				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 3,
						this.normMatrixEntryAlpha( -this.alphas[snakePointNum - 2]) 
          - this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 2]));
				// oldVal= A.get(2 * snakePointNum - 2, 2 * snakePointNum - 2);
				// A.set(2 * snakePointNum - 2, 2 * snakePointNum - 2,
				// oldVal+2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 2,
//						this.normMatrixEntry( 2 * this.alphas[snakePointNum - 2] 
//						                    + 6 * this.betas[snakePointNum - 2]));
				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 2,
						this.normMatrixEntryAlpha( 2 * this.alphas[snakePointNum - 2]) 
					+ this.normMatrixEntryBeta(6 * this.betas[snakePointNum - 2]));
				// oldVal= A.get(2 * snakePointNum - 2, 2 * snakePointNum - 1);
				// A.set(2 * snakePointNum - 2, 2 * snakePointNum - 1,
				// oldVal-this.alphaInitial - 4 * this.betaInitial);
//				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 1,
//						this.normMatrixEntry(  -this.alphas[snakePointNum - 2] - 
//								                4 * this.betas[snakePointNum - 2]));
				A.set(2 * snakePointNum - 2, 2 * snakePointNum - 1,
						this.normMatrixEntryAlpha(  -this.alphas[snakePointNum - 2]) 
					- this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 2]));
				// last line
				// oldVal= A.get(2 * snakePointNum - 1, snakePointNum);
				// A.set(2 * snakePointNum - 1, snakePointNum,
				// oldVal-this.alphaInitial - 4 * this.betaInitial);
//				A.set(2 * snakePointNum - 1, snakePointNum,
//						this.normMatrixEntry( -this.alphas[snakePointNum - 1]
//				                     - 4 * this.betas[snakePointNum - 1]));
				A.set(2 * snakePointNum - 1, snakePointNum,
						this.normMatrixEntryAlpha( -this.alphas[snakePointNum - 1])
          - this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 1]));
				// oldVal= A.get(2 * snakePointNum - 1, snakePointNum + 1);
				// A.set(2 * snakePointNum - 1, snakePointNum + 1, oldVal+this.betaInitial);
//				A.set(2 * snakePointNum - 1, snakePointNum + 1,
//						this.normMatrixEntry( this.betas[snakePointNum - 1]));
				A.set(2 * snakePointNum - 1, snakePointNum + 1,
						this.normMatrixEntryBeta( this.betas[snakePointNum - 1]));
				// oldVal= A.get(2 * snakePointNum - 1, 2 * snakePointNum - 3);
				// A.set(2 * snakePointNum - 1, 2 * snakePointNum - 3,
				// oldVal+this.betaInitial);
//				A.set(2 * snakePointNum - 1, 2 * snakePointNum - 3,
//						this.normMatrixEntry(  this.betas[snakePointNum - 1]));
				A.set(2 * snakePointNum - 1, 2 * snakePointNum - 3,
						this.normMatrixEntryBeta(  this.betas[snakePointNum - 1]));
				// oldVal= A.get(2 * snakePointNum - 1, 2 * snakePointNum - 2);
				// A.set(2 * snakePointNum - 1, 2 * snakePointNum - 2,
				// oldVal-this.alphaInitial - 4 * this.betaInitial);
//				A.set(2 * snakePointNum - 1, 2 * snakePointNum - 2,
//						this.normMatrixEntry(  -this.alphas[snakePointNum - 1] 
//						                  - 4 * this.betas[snakePointNum - 1]));
				A.set(2 * snakePointNum - 1, 2 * snakePointNum - 2,
						this.normMatrixEntryAlpha(  -this.alphas[snakePointNum - 1]) 
					- this.normMatrixEntryBeta(4 * this.betas[snakePointNum - 1]));
				// oldVal= A.get(2 * snakePointNum - 1, 2 * snakePointNum - 1);
				// A.set(2 * snakePointNum - 1, 2 * snakePointNum - 1,
				// oldVal + 2 * this.alphaInitial + 6 * this.betaInitial);
//				A.set(2 * snakePointNum - 1, 2 * snakePointNum - 1,
//						this.normMatrixEntry( 2 * this.alphas[snakePointNum - 1] 
//						                    + 6 * this.betas[snakePointNum - 1]));
				A.set(2 * snakePointNum - 1, 2 * snakePointNum - 1,
						this.normMatrixEntryAlpha( 2 * this.alphas[snakePointNum - 1]) 
          + this.normMatrixEntryBeta(6 * this.betas[snakePointNum - 1]));
				return A;
		}

		private double normMatrixEntryAlpha(double v) {
			if (   !this.normalizationImpossible 
					&& Math.abs(this.energyRangeAlpha) > MTBConstants.epsilon)
				return (v - this.minEnergyAlpha)/(this.energyRangeAlpha) * 
										this.targetEnergyRange + this.targetEnergyMin;
			return v;
		}

		private double normMatrixEntryBeta(double v) {
			if (   !this.normalizationImpossible 
					&& Math.abs(this.energyRangeBeta) > MTBConstants.epsilon)
				return (v - this.minEnergyBeta)/(this.energyRangeBeta) * 
										this.targetEnergyRange + this.targetEnergyMin;
			return v;
		}

//		private double normMatrixEntry(double v) {
//			if (   !this.normalizationImpossible 
//					&& Math.abs(this.energyRange) > MTBConstants.epsilon)
//				return (v - this.minEnergy)/(this.energyRange) * 
//										this.targetEnergyRange + this.targetEnergyMin;
//			return v;
//		}
		
		@Override
		public Matrix getDerivative_VectorPart(SnakeOptimizerSingleVarCalc opt) {
				return null;
		}

		@Override
		public String toString() {
				return new String("MTBSnkEner: Kass Length-Curvature Energy " + "(alpha = "
				    + this.alphaInitial + " , beta = " + this.betaInitial + ")");
		}

		@Override
		public void updateStatus(SnakeOptimizerSingleVarCalc o) {
				// nothing to do here...
		}
		@Override
		public void updateStatus(SnakeOptimizerSingleGreedy o) {
				// nothing to do here...
		}
}
