/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5679 $
 * $Date: 2012-06-22 11:41:56 +0200 (Fr, 22. Jun 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.snakes.energies.paramAdapt;

import java.util.Vector;

import de.unihalle.informatik.Alida.annotations.ALDClassParameter;
import de.unihalle.informatik.Alida.annotations.ALDDerivedClass;
import de.unihalle.informatik.Alida.annotations.ALDParametrizedClass;
import de.unihalle.informatik.MiToBo.segmentation.snakes.datatypes.MTBSnake;
import de.unihalle.informatik.MiToBo.segmentation.snakes.datatypes.MTBSnakePoint2D;
import de.unihalle.informatik.MiToBo.segmentation.snakes.energies.MTBSnakeEnergyDerivable;
import de.unihalle.informatik.MiToBo.segmentation.snakes.energies.MTBSnakeEnergyCDIB_Distance;
import de.unihalle.informatik.MiToBo.segmentation.snakes.optimize.SnakeOptimizerSingle;
import de.unihalle.informatik.MiToBo.segmentation.snakes.optimize.SnakeOptimizerSingleVarCalc;

/**
 * Parameter adaptation for Kass et al. internal energy using a fixed value to
 * decrease the values of alpha and beta.
 * 
 * 
 * @author Danny Misiak
 * @deprecated
 */


@ALDDerivedClass
@ALDParametrizedClass
@Deprecated
public class MTBSnakeEnergyCD_KassInt_ParamAdaptFix extends
    MTBSnakeEnergyCD_KassInt_ParamAdapt {

	@ALDClassParameter(label="Percentual Alpha Decrease")
		private double alphaDecrease;
	@ALDClassParameter(label="Percentual Beta Decrease")
		private double betaDecrease;
	@ALDClassParameter(label="Initial Alpha")
		private double initAlpha;
	@ALDClassParameter(label="Initial Beta")
		private double initBeta;

	/**
	 * Default constructor.
	 */
	public MTBSnakeEnergyCD_KassInt_ParamAdaptFix() {
		// nothing to do here
	}

	/**
		 * Construct a new updater for the internal energies using a fixed value to
		 * decrease alpha and beta.
		 * 
		 */
		public MTBSnakeEnergyCD_KassInt_ParamAdaptFix(double initAlpha,
		    double initBeta, double alphaDecrease, double betaDecrease) {
				this.initAlpha = initAlpha;
				this.initBeta = initBeta;
				this.alphaDecrease = alphaDecrease;
				this.betaDecrease = betaDecrease;
		}

		/**
		 * Get the fix alpha decrease value.
		 * 
		 * @return Alpha decrease value.
		 */
		public double getAlphaDecrease() {
				return this.alphaDecrease;
		}

		/**
		 * Get the fix beta decrease value.
		 * 
		 * @return Beta decrease value.
		 */
		public double getBetaDecrease() {
				return this.betaDecrease;
		}

		@Override
		public double[] alphaUpdate(SnakeOptimizerSingleVarCalc o, double[] curAlphas) {
				System.out
				    .println(" --------- > update \u03B1 fix @ " + this.alphaDecrease);
				MTBSnake s = o.getCurrentSnake();
				int count = o.getIterationCount();
				int pNum = s.getPointNum();
				if (s != null && pNum > 0) {
						/*
						 * External energy, which is used by the current snake optimization and
						 * should be used in the gamma adaptation.
						 */
						Vector<MTBSnakeEnergyDerivable> energies = o.getEnergies();
						MTBSnakeEnergyCDIB_Distance extEnergy = null;
						for (int i = 0; i < energies.size(); i++) {
								if (energies.get(i) instanceof MTBSnakeEnergyCDIB_Distance) {
										extEnergy = (MTBSnakeEnergyCDIB_Distance) energies.get(i);
										break;
								}
						}
						double[] newAlphas = new double[pNum];
						Vector<MTBSnakePoint2D> points = s.getSnakePoints();

						int[] index = { -5, -4 - 3, -2, -1, 0, 1, 2, 3, 4, 5 };

						for (int i = 0; i < points.size(); i++) {
								// get current snake point
								MTBSnakePoint2D curPoint = s.getSnakePoints().elementAt(i);
								/*
								 * Do gamma adaptation dependent on external energy for the current
								 * snake point.
								 */

								double eValue = extEnergy.getValue_norm(curPoint.x, curPoint.y);

								if (eValue < 0.05) {

										double sum = 0;
										double var = 0.0;

										for (int j = 0; j < index.length; j++) {
												MTBSnakePoint2D p;
												if ((i + index[j]) < 0) {
														p = points.elementAt(points.size() + (i + index[j]));
												} else if ((i + index[j]) > (points.size() - 1)) {
														p = points.elementAt((i + index[j]) - points.size());
												} else {
														p = points.elementAt(i + index[j]);
												}
												sum += extEnergy.getValue_norm(p.x, p.y);

										}

										double mean = sum / (double) index.length;
										for (int j = 0; j < index.length; j++) {
												MTBSnakePoint2D p;
												if ((i + index[j]) < 0) {
														p = points.elementAt(points.size() + (i + index[j]));
												} else if ((i + index[j]) > (points.size() - 1)) {
														p = points.elementAt((i + index[j]) - points.size());
												} else {
														p = points.elementAt(i + index[j]);
												}
												var += (extEnergy.getValue_norm(p.x, p.y) - mean)
												    * (extEnergy.getValue_norm(p.x, p.y) - mean);
										}
										var = var / (double) index.length;
										if (var > 1.0e-5) {
												if (curPoint.getOldId() >= 0) {
														if (curAlphas[curPoint.getOldId()] >= this.alphaDecrease) {
																newAlphas[i] = curAlphas[curPoint.getOldId()]
																    - this.alphaDecrease;
														} else {
																newAlphas[i] = 0.0;
														}
												}
												newAlphas[i] = 0.0;
										} else {
												newAlphas[i] = 0.0;

										}

								}

						}

						return newAlphas;

						// double[] newAlphas = new double[pNum];
						// for (int i = 0; i < pNum; ++i) {
						// if (curAlphas[0] > this.alphaDecrease) {
						// newAlphas[i] = curAlphas[0] - this.alphaDecrease;
						// } else {
						// newAlphas[i] = 1e-4;
						// }
						// }
						// return newAlphas;
				}

				return null;
		}

		// double[] newAlphas = new double[pNum];
		// for (int i = 0; i < pNum; ++i) {
		// newAlphas[i] = this.initAlpha
		// - (Math.pow((Math.pow((initAlpha - curAlphas[0]),
		// (1 / this.alphaDecrease)) + 1), this.alphaDecrease));
		// if (newAlphas[i] < 0.0) {
		// newAlphas[i] = 0.0;
		// }
		// }
		// return newAlphas;

		@Override
		public double[] betaUpdate(SnakeOptimizerSingleVarCalc o, double[] curBetas) {
				System.out.println(" --------- > update \u03B2 fix @ " + this.betaDecrease);
				MTBSnake s = o.getCurrentSnake();
				int pNum = s.getPointNum();
				if (s != null && pNum > 0) {
						double[] newBetas = new double[pNum];
						for (int i = 0; i < pNum; ++i) {
								if (curBetas[0] > this.betaDecrease) {
										newBetas[i] = curBetas[0] - this.betaDecrease;
								} else {
										newBetas[i] = 0.0;
								}
						}
						return newBetas;
				}
				return null;
		}

		@Override
		public Double getMaxAlpha() {
				return (new Double(this.initAlpha));
		}

		@Override
		public Double getMaxBeta() {
				return (new Double(this.initBeta));
		}

		@Override
		public Double getMinAlpha() {
				return (new Double(0.0));
		}

		@Override
		public Double getMinBeta() {
				return (new Double(0.0));
		}

		@Override
		public String toString() {
				return new String("Kass_ParamUpdate: Fixed Decrease");
		}

}
