/*
 * This file is part of MiToBo, the Microscope Image Analysis Toolbox.
 *
 * Copyright (C) 2010 - 2012
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Fore more information on MiToBo, visit
 *
 *    http://www.informatik.uni-halle.de/mitobo/
 *
 */

/* 
 * Most recent change(s):
 * 
 * $Rev: 5679 $
 * $Date: 2012-06-22 11:41:56 +0200 (Fr, 22. Jun 2012) $
 * $Author: moeller $
 * 
 */

package de.unihalle.informatik.MiToBo.segmentation.snakes.energies.paramAdapt;



import de.unihalle.informatik.Alida.annotations.ALDDerivedClass;
import de.unihalle.informatik.Alida.annotations.ALDParametrizedClass;
import de.unihalle.informatik.MiToBo.segmentation.snakes.datatypes.MTBSnake;
import de.unihalle.informatik.MiToBo.segmentation.snakes.optimize.*;

/**
 * Dummy parameter adaptation for Kass et al. internal energy.
 * <p>
 * This class simply adapts the size of the vector, assuming the first 
 * value as the default ones.
 * 
 * @author moeller
 * @deprecated
 */
@ALDDerivedClass
@ALDParametrizedClass

@Deprecated
public class MTBSnakeEnergyCD_KassInt_ParamAdaptNone extends
    MTBSnakeEnergyCD_KassInt_ParamAdapt {

	/**
	 * Initial alpha value of the internal Kass energy.
	 */
	private transient double initAlpha;
	/**
	 * Initial beta value of the internal Kass energy.
	 */
	private transient double initBeta;

	/**
	 * Default constructor.
	 */
	public MTBSnakeEnergyCD_KassInt_ParamAdaptNone() {
		// nothing to do here
	}

	/**
	 * Constructor with parameters.
	 * <p>
	 * Constructs a new updater for internal energies using the initial alpha
	 * and beta parameter values of the Kass internal energy to set the maximum
	 * and minimum range of the alpha and beta values.
	 */
	public MTBSnakeEnergyCD_KassInt_ParamAdaptNone(double _iAlpha, double _iBeta) {
		this.initAlpha = _iAlpha;
		this.initBeta = _iBeta;
	}

	@SuppressWarnings("null")
	@Override
	public double[] alphaUpdate(SnakeOptimizerSingleVarCalc o, double[] alphas) {
		MTBSnake s = o.getCurrentSnake();
		int pNum = s.getPointNum();
		if (s != null && pNum > 0) {
			double[] newAlphas = new double[pNum];
			for (int i = 0; i < pNum; ++i) {
				newAlphas[i] = alphas[0];
			}
			return newAlphas;
		}
		return null;
	}

	@SuppressWarnings("null")
	@Override
	public double[] betaUpdate(SnakeOptimizerSingleVarCalc o, double[] betas) {
		MTBSnake s = o.getCurrentSnake();
		int pNum = s.getPointNum();
		if (s != null && pNum > 0) {
			double[] newBetas = new double[pNum];
			for (int i = 0; i < pNum; ++i) {
				newBetas[i] = betas[0];
			}
			return newBetas;
		}
		return null;
	}

	@Override
	public Double getMaxAlpha() {
		return (new Double(this.initAlpha));
	}

	@Override
	public Double getMaxBeta() {
		return (new Double(this.initBeta));
	}

	@Override
	public Double getMinAlpha() {
		return (new Double(this.initAlpha));
	}

	@Override
	public Double getMinBeta() {
		return (new Double(this.initBeta));
	}

	@Override
	public String toString() {
		return new String("Kass_ParamUpdate: none");
	}
}
